@@ -16,9 +16,11 @@
 
 // @author: Logan Evans <lpe@fb.com>
 
-#include <stdlib.h>
+#include <cstddef>
 
-extern "C" {
-void* __folly_memcpy(
-    void* __restrict dst, const void* __restrict src, size_t size);
-}
+namespace folly {
+
+extern "C" void* __folly_memcpy(
+    void* __restrict dst, const void* __restrict src, std::size_t size);
+
+} // namespace folly
