@@ -20,10 +20,11 @@
  */
 
 /**
- * @class Function
+ * @class folly::Function
+ * @refcode folly/docs/examples/folly/Function.cpp
  *
- * @brief A polymorphic function wrapper that is not copyable and does not
- *    require the wrapped function to be copy constructible.
+ * A polymorphic function wrapper that is not copyable and does not
+ * require the wrapped function to be copy constructible.
  *
  * `folly::Function` is a polymorphic function wrapper, similar to
  * `std::function`. The template parameters of the `folly::Function` define
@@ -196,24 +197,6 @@
  * select a non-const invoke operator (or the const one if no non-const one is
  * present), and then move it into a const `folly::Function` using
  * `constCastFunction`.
- * The name of `constCastFunction` should warn you that something
- * potentially dangerous is happening. As a matter of fact, using
- * `std::function` always involves this potentially dangerous aspect, which
- * is why it is not considered fully const-safe or even const-correct.
- * However, in most of the cases you will not need the dangerous aspect at all.
- * Either you do not require invocation of the function from a const context,
- * in which case you do not need to use `constCastFunction` and just
- * use the inner `folly::Function` in the example above, i.e. just use a
- * non-const `folly::Function`. Or, you may need invocation from const, but
- * the callable you are wrapping does not mutate its state (e.g. it is a class
- * object and implements `operator() const`, or it is a normal,
- * non-mutable lambda), in which case you can wrap the callable in a const
- * `folly::Function` directly, without using `constCastFunction`.
- * Only if you require invocation from a const context of a callable that
- * may mutate itself when invoked you have to go through the above procedure.
- * However, in that case what you do is potentially dangerous and requires
- * the equivalent of a `const_cast`, hence you need to call
- * `constCastFunction`.
  */
 
 #pragma once
@@ -279,6 +262,10 @@ constexpr bool isEmptyFunction(T const& t) {
 template <typename F, typename... Args>
 using CallableResult = decltype(FOLLY_DECLVAL(F &&)(FOLLY_DECLVAL(Args &&)...));
 
+template <typename F, typename... Args>
+constexpr bool CallableNoexcept =
+    noexcept(FOLLY_DECLVAL(F&&)(FOLLY_DECLVAL(Args&&)...));
+
 template <
     typename From,
     typename To,
@@ -301,7 +288,7 @@ template <typename T>
 using CallArg = conditional_t<is_register_pass_v<T>, T, T&&>;
 #endif
 
-template <typename F, typename R, typename... A>
+template <typename F, bool Nx, typename R, typename... A>
 class FunctionTraitsSharedProxy {
   std::shared_ptr<Function<F>> sp_;
 
@@ -310,7 +297,7 @@ class FunctionTraitsSharedProxy {
   explicit FunctionTraitsSharedProxy(Function<F>&& func)
       : sp_(func ? std::make_shared<Function<F>>(std::move(func))
                  : std::shared_ptr<Function<F>>()) {}
-  R operator()(A... args) const {
+  R operator()(A... args) const noexcept(Nx) {
     if (!sp_) {
       throw_exception<std::bad_function_call>();
     }
@@ -320,24 +307,20 @@ class FunctionTraitsSharedProxy {
   explicit operator bool() const noexcept { return sp_ != nullptr; }
 
   friend bool operator==(
-      FunctionTraitsSharedProxy<F, R, A...> const& proxy,
-      std::nullptr_t) noexcept {
+      FunctionTraitsSharedProxy const& proxy, std::nullptr_t) noexcept {
     return proxy.sp_ == nullptr;
   }
   friend bool operator!=(
-      FunctionTraitsSharedProxy<F, R, A...> const& proxy,
-      std::nullptr_t) noexcept {
+      FunctionTraitsSharedProxy const& proxy, std::nullptr_t) noexcept {
     return proxy.sp_ != nullptr;
   }
 
   friend bool operator==(
-      std::nullptr_t,
-      FunctionTraitsSharedProxy<F, R, A...> const& proxy) noexcept {
+      std::nullptr_t, FunctionTraitsSharedProxy const& proxy) noexcept {
     return proxy.sp_ == nullptr;
   }
   friend bool operator!=(
-      std::nullptr_t,
-      FunctionTraitsSharedProxy<F, R, A...> const& proxy) noexcept {
+      std::nullptr_t, FunctionTraitsSharedProxy const& proxy) noexcept {
     return proxy.sp_ != nullptr;
   }
 };
@@ -393,7 +376,7 @@ struct FunctionTraits<ReturnType(Args...)> {
   }
 
   using SharedProxy =
-      FunctionTraitsSharedProxy<NonConstSignature, ReturnType, Args...>;
+      FunctionTraitsSharedProxy<NonConstSignature, false, ReturnType, Args...>;
 };
 
 template <typename ReturnType, typename... Args>
@@ -444,7 +427,7 @@ struct FunctionTraits<ReturnType(Args...) const> {
   }
 
   using SharedProxy =
-      FunctionTraitsSharedProxy<ConstSignature, ReturnType, Args...>;
+      FunctionTraitsSharedProxy<ConstSignature, false, ReturnType, Args...>;
 };
 
 #if FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE
@@ -455,7 +438,10 @@ struct FunctionTraits<ReturnType(Args...) noexcept> {
   using NonConstSignature = ReturnType(Args...) noexcept;
   using OtherSignature = ConstSignature;
 
-  template <typename F, typename R = CallableResult<F&, Args...>>
+  template <
+      typename F,
+      typename R = CallableResult<F&, Args...>,
+      std::enable_if_t<CallableNoexcept<F&, Args...>, int> = 0>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
   template <typename Fun>
@@ -496,7 +482,7 @@ struct FunctionTraits<ReturnType(Args...) noexcept> {
   }
 
   using SharedProxy =
-      FunctionTraitsSharedProxy<NonConstSignature, ReturnType, Args...>;
+      FunctionTraitsSharedProxy<NonConstSignature, true, ReturnType, Args...>;
 };
 
 template <typename ReturnType, typename... Args>
@@ -506,7 +492,10 @@ struct FunctionTraits<ReturnType(Args...) const noexcept> {
   using NonConstSignature = ReturnType(Args...) noexcept;
   using OtherSignature = NonConstSignature;
 
-  template <typename F, typename R = CallableResult<const F&, Args...>>
+  template <
+      typename F,
+      typename R = CallableResult<const F&, Args...>,
+      std::enable_if_t<CallableNoexcept<const F&, Args...>, int> = 0>
   using IfSafeResult = IfSafeResultImpl<R, ReturnType>;
 
   template <typename Fun>
@@ -547,7 +536,7 @@ struct FunctionTraits<ReturnType(Args...) const noexcept> {
   }
 
   using SharedProxy =
-      FunctionTraitsSharedProxy<ConstSignature, ReturnType, Args...>;
+      FunctionTraitsSharedProxy<ConstSignature, true, ReturnType, Args...>;
 };
 #endif
 
@@ -727,11 +716,7 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
 
   /**
    * Constructs a new `Function` from any callable object that is _not_ a
-   * `folly::Function`. This handles function pointers, pointers to static
-   * member functions, `std::reference_wrapper` objects, `std::function`
-   * objects, and arbitrary objects that implement `operator()` if the parameter
-   * signature matches (i.e. it returns an object convertible to `R` when called
-   * with `Args...`).
+   * `folly::Function`.
    *
    * \note `typename Traits::template IfSafeResult<Fun>` prevents this overload
    * from being selected by overload resolution when `fun` is not a compatible
@@ -744,6 +729,12 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
    * checked in `IsSmall`, and shouldn't be, because once the `Function` is
    * constructed, the contained object is never copied. This check is for this
    * ctor only, in the case that this ctor does a copy.
+   *
+   * @param fun function pointers, pointers to static
+   *     member functions, `std::reference_wrapper` objects, `std::function`
+   *     objects, and arbitrary objects that implement `operator()` if the
+   * parameter signature matches (i.e. it returns an object convertible to `R`
+   * when called with `Args...`).
    */
   template <
       typename Fun,
@@ -778,6 +769,7 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
 
   /**
    * For move-constructing from a `folly::Function<X(Ys...) [const?]>`.
+   *
    * For a `Function` with a `const` function type, the object must be
    * callable from a `const`-reference, i.e. implement `operator() const`.
    * For a `Function` with a non-`const` function type, the object will
@@ -833,9 +825,11 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
    */
   Function& operator=(Function&& that) noexcept {
     exec(Op::NUKE, &data_, nullptr);
-    that.exec(Op::MOVE, &that.data_, &data_);
-    exec_ = that.exec_;
-    call_ = that.call_;
+    if (FOLLY_LIKELY(this != &that)) {
+      that.exec(Op::MOVE, &that.data_, &data_);
+      exec_ = that.exec_;
+      call_ = that.call_;
+    }
     that.exec_ = nullptr;
     that.call_ = &Traits::uninitCall;
     return *this;
@@ -903,6 +897,8 @@ class Function final : private detail::function::FunctionTraits<FunctionType> {
 
   /**
    * Exchanges the callable objects of `*this` and `that`.
+   *
+   * @param that a folly::Function ref
    */
   void swap(Function& that) noexcept { std::swap(*this, that); }
 
@@ -965,8 +961,27 @@ bool operator!=(std::nullptr_t, const Function<FunctionType>& fn) {
 }
 
 /**
- * NOTE: See detailed note about `constCastFunction` at the top of the file.
- * This is potentially dangerous and requires the equivalent of a `const_cast`.
+ * Casts a `folly::Function` from non-const to a const signature.
+ *
+ * NOTE: The name of `constCastFunction` should warn you that something
+ * potentially dangerous is happening. As a matter of fact, using
+ * `std::function` always involves this potentially dangerous aspect, which
+ * is why it is not considered fully const-safe or even const-correct.
+ * However, in most of the cases you will not need the dangerous aspect at all.
+ * Either you do not require invocation of the function from a const context,
+ * in which case you do not need to use `constCastFunction` and just
+ * use a non-const `folly::Function`. Or, you may need invocation from const,
+ * but the callable you are wrapping does not mutate its state (e.g. it is a
+ * class object and implements `operator() const`, or it is a normal,
+ * non-mutable lambda), in which case you can wrap the callable in a const
+ * `folly::Function` directly, without using `constCastFunction`.
+ * Only if you require invocation from a const context of a callable that
+ * may mutate itself when invoked you have to go through the above procedure.
+ * However, in that case what you do is potentially dangerous and requires
+ * the equivalent of a `const_cast`, hence you need to call
+ * `constCastFunction`.
+ *
+ * @param that a non-const folly::Function.
  */
 template <typename ReturnType, typename... Args>
 Function<ReturnType(Args...) const> constCastFunction(
@@ -997,9 +1012,9 @@ Function<ReturnType(Args...) const noexcept> constCastFunction(
 #endif
 
 /**
- * @class FunctionRef
+ * @class folly::FunctionRef
  *
- * @brief A reference wrapper for callable objects
+ * A reference wrapper for callable objects
  *
  * FunctionRef is similar to std::reference_wrapper, but the template parameter
  * is the function signature type rather than the type of the referenced object.
