@@ -14,6 +14,16 @@
  * limitations under the License.
  */
 
+/**
+ * Provides a unified interface for IP addresses.
+ *
+ * @refcode folly/docs/examples/folly/ipaddress.cpp
+ *
+ * @class folly::IPAddress
+ * @see IPAddressV6
+ * @see IPAddressV4
+ */
+
 #pragma once
 
 #include <functional>
@@ -40,33 +50,6 @@ class IPAddress;
  */
 typedef std::pair<IPAddress, uint8_t> CIDRNetwork;
 
-/**
- * Provides a unified interface for IP addresses.
- *
- * @note If you compare 2 IPAddress instances, v4-to-v6-mapped addresses are
- * compared as V4 addresses.
- *
- * @note toLong/fromLong deal in network byte order, use toLongHBO/fromLongHBO
- * if working in host byte order.
- *
- * Example usage:
- * @code
- *   IPAddress v4addr("192.0.2.129");
- *   IPAddress v6map("::ffff:192.0.2.129");
- *   CHECK(v4addr.inSubnet("192.0.2.0/24") ==
- *         v4addr.inSubnet(IPAddress("192.0.2.0"), 24));
- *   CHECK(v4addr.inSubnet("192.0.2.128/30"));
- *   CHECK(!v4addr.inSubnet("192.0.2.128/32"));
- *   CHECK(v4addr.asV4().toLong() == 2164392128);
- *   CHECK(v4addr.asV4().toLongHBO() == 3221226113);
- *   CHECK(v4addr.isV4());
- *   CHECK(v6addr.isV6());
- *   CHECK(v4addr == v6map);
- *   CHECK(v6map.isIPv4Mapped());
- *   CHECK(v4addr.asV4() == IPAddress::createIPv4(v6map));
- *   CHECK(IPAddress::createIPv6(v4addr) == v6map.asV6());
- * @encode
- */
 class IPAddress {
  private:
   template <typename F>
@@ -122,20 +105,29 @@ class IPAddress {
   }
 
  public:
-  // returns true iff the input string can be parsed as an ip-address
+  /**
+   * Returns true if the input string can be parsed as an IP address.
+   */
   static bool validate(StringPiece ip) noexcept;
-
-  // return the V4 representation of the address, converting it from V6 to V4 if
-  // needed. Note that this will throw an IPAddressFormatException if the V6
-  // address is not IPv4Mapped.
+  /**
+   * Return the IPAddressV4 representation of the address, converting
+   * from V6 to V4 if needed.
+   *
+   *
+   *
+   * @throws IPAddressFormatException if the V6 Address is not IPv4 mappes
+   */
   static IPAddressV4 createIPv4(const IPAddress& addr);
 
-  // return the V6 representation of the address, converting it from V4 to V6 if
-  // needed.
+  /**
+   * Return the address as a IPAddressV6, converting from V4 to V6 if
+   * needed.
+   */
   static IPAddressV6 createIPv6(const IPAddress& addr);
 
   /**
    * Create a network and mask from a CIDR formatted address string.
+   *
    * @param [in] ipSlashCidr IP/CIDR formatted string to split
    * @param [in] defaultCidr default value if no /N specified (if defaultCidr
    *             is -1, will use /32 for IPv4 and /128 for IPv6)
@@ -149,19 +141,20 @@ class IPAddress {
 
   /**
    * Create a network and mask from a CIDR formatted address string.
-   * Same as tryCreateNetwork() but throws IPAddressFormatException on error.
-   * The implementation calls tryCreateNetwork(...) underneath
    *
-   * @throws IPAddressFormatException if invalid address
+   * Same as tryCreateNetwork() but throws on error.
+   *
+   * @throws IPAddressFormatException
    * @return pair with IPAddress network and uint8_t mask
    */
   static CIDRNetwork createNetwork(
       StringPiece ipSlashCidr, int defaultCidr = -1, bool mask = true);
 
   /**
-   * Return a string representation of a CIDR block created with createNetwork.
-   * @param [in] network pair of address and cidr
+   * Return a string representation of a CIDR block created with
+   * createNetwork().
    *
+   * @param [in] network pair of address and cidr
    * @return string representing the netblock
    */
   static std::string networkToString(const CIDRNetwork& network);
@@ -169,7 +162,9 @@ class IPAddress {
   /**
    * Create a new IPAddress instance from the provided binary data
    * in network byte order.
-   * @throws IPAddressFormatException if len is not 4 or 16
+   *
+   * @throws IPAddressFormatException if the length of `bytes` is not 4 or 16.
+   *
    */
   static IPAddress fromBinary(ByteRange bytes);
 
@@ -181,22 +176,32 @@ class IPAddress {
       ByteRange bytes) noexcept;
 
   /**
-   * Tries to create a new IPAddress instance from provided string and
-   * returns it on success. Returns IPAddressFormatError on failure.
+   * Tries to create a new IPAddress instance from provided string.
+   *
+   * On failure, returns IPAddressFormatError.
    */
   static Expected<IPAddress, IPAddressFormatError> tryFromString(
       StringPiece str) noexcept;
 
   /**
-   * Create an IPAddress from a 32bit long (network byte order).
-   * @throws IPAddressFormatException
+   * Create an IPAddress from a `uint32_t`, using network byte order.
+   *
+   * @throws IPAddressFormatException if `src` does not represent a valid IP
+   * Address.
    */
   static IPAddress fromLong(uint32_t src);
-  // Same as above, but host byte order
+
+  /**
+   * Create an IPAddress from a `uint32_t`, using host byte order.
+   *
+   * @throws IPAddressFormatException if `src` does not represent a valid IP
+   * address.
+   */
   static IPAddress fromLongHBO(uint32_t src);
 
-  // Given 2 IPAddress,mask pairs extract the longest common IPAddress,
-  // mask pair
+  /**
+   * Given 2 (IPAddress, mask) pairs, extract the longest common pair.
+   */
   static CIDRNetwork longestCommonPrefix(
       const CIDRNetwork& one, const CIDRNetwork& two);
 
@@ -208,41 +213,63 @@ class IPAddress {
   /**
    * Parse an IPAddress from a string representation.
    *
-   * Formats accepted are exactly the same as the ones accepted by inet_pton(),
-   * using AF_INET6 if the string contains colons, and AF_INET otherwise;
-   * with the exception that the whole address can optionally be enclosed
-   * in square brackets.
+   * Formats accepted are exactly the same as the ones accepted by
+   * `inet_pton()`, using `AF_INET6` if the string contains colons, and `AF_INET
+   * `otherwise; with the exception that the whole address can optionally be
+   * enclosed in square brackets.
    *
-   * @throws IPAddressFormatException
+   * @throws IPAddressFormatException if `str` is not a valid IP Address.
    */
   explicit IPAddress(StringPiece str);
 
   /**
-   * Create an IPAddress from a sockaddr.
-   * @throws IPAddressFormatException if nullptr or not AF_INET or AF_INET6
+   * Create an IPAddress from a `sockaddr` struct.
+   *
+   * @throws IPAddressFormatException if `addr` is a nullptr, or not `AF_INET`
+   * or `AF_INET6`
    */
   explicit IPAddress(const sockaddr* addr);
 
-  // Create an IPAddress from a V4 address
+  /**
+   * Create an IPAddress from an IPAddressV4
+   */
   /* implicit */ IPAddress(const IPAddressV4 ipV4Addr) noexcept;
+
+  /**
+   * Create an IPAddress from an `in_addr` representation of an IPV4 address
+   */
   /* implicit */ IPAddress(const in_addr addr) noexcept;
 
-  // Create an IPAddress from a V6 address
+  /**
+   * Create an IPAddress from an IPAddressV6
+   */
   /* implicit */ IPAddress(const IPAddressV6& ipV6Addr) noexcept;
+
+  /**
+   * Create an IPAddress from an `in6_addr` representation of an IPV6 address
+   */
   /* implicit */ IPAddress(const in6_addr& addr) noexcept;
 
-  // Assign from V4 address
+  /**
+   * @overloadbrief Copy assignment from other IPAddress representations
+   *
+   * Copy assignment from an IPAddressV4
+   */
   IPAddress& operator=(const IPAddressV4& ipv4_addr) noexcept;
 
-  // Assign from V6 address
+  /**
+   * Copy assignment from an IPAddressV6
+   */
   IPAddress& operator=(const IPAddressV6& ipv6_addr) noexcept;
 
   /**
    * Converts an IPAddress to an IPAddressV4 instance.
+   *
    * @note This is not some handy convenience wrapper to convert an IPv4 address
-   *       to a mapped IPv6 address. If you want that use
-   *       IPAddress::createIPv6(addr)
-   * @throws InvalidAddressFamilyException is not a V4 instance
+   *       to a mapped IPv6 address. If you want that use createIPv6()
+   *
+   * @throws InvalidAddressFamilyException if the IP Address is not currently
+   * a valid V4 instance.
    */
   const IPAddressV4& asV4() const {
     if (UNLIKELY(!isV4())) {
@@ -252,8 +279,10 @@ class IPAddress {
   }
 
   /**
-   * Converts an IPAddress to an IPAddressV6 instance.
-   * @throws InvalidAddressFamilyException is not a V6 instance
+   * Converts an IPAddress to an IPAddressV6 instance
+   *
+   * @throws InvalidAddressFamilyException if the IP Address is not currently
+   * a valid V6 instance.
    */
   const IPAddressV6& asV6() const {
     if (UNLIKELY(!isV6())) {
@@ -262,10 +291,18 @@ class IPAddress {
     return addr_.ipV6Addr;
   }
 
-  // Return sa_family_t of IPAddress
+  /**
+   * Return then `sa_family_t` of the IP Address
+   */
   sa_family_t family() const { return family_; }
 
-  // Populate sockaddr_storage with an appropriate value
+  /**
+   * Given a `sockaddr_storage` struct, populate it with an appropriate value.
+   *
+   * @param [out] dest The struct to populate
+   * @param port The port number to put in the struct (defaults to 0)
+   *
+   */
   int toSockaddrStorage(sockaddr_storage* dest, uint16_t port = 0) const {
     if (dest == nullptr) {
       throw_exception<IPAddressFormatException>("dest must not be null");
@@ -296,7 +333,8 @@ class IPAddress {
   }
 
   /**
-   * Check if the address is found in the specified CIDR netblock.
+   * @overloadbrief Check if the address is found in the specified CIDR
+   * netblock.
    *
    * This will return false if the specified cidrNet is V4, but the address is
    * V6. It will also return false if the specified cidrNet is V6 but the
@@ -306,7 +344,7 @@ class IPAddress {
    * @note This is slower than the below counterparts. If perf is important use
    *       one of the two argument variations below.
    * @param [in] cidrNetwork address in "192.168.1.0/24" format
-   * @throws IPAddressFormatException if no /mask
+   * @throws IPAddressFormatException if no /mask in cidrNetwork
    * @return true if address is part of specified subnet with cidr
    */
   bool inSubnet(StringPiece cidrNetwork) const;
@@ -321,6 +359,7 @@ class IPAddress {
 
   /**
    * Check if an IPAddress belongs to the subnet with the given mask.
+   *
    * This is the same as inSubnet but the mask is provided instead of looked up
    * from the cidr.
    * @param [in] subnet Subnet to check against
@@ -329,80 +368,123 @@ class IPAddress {
    */
   bool inSubnetWithMask(const IPAddress& subnet, ByteRange mask) const;
 
-  // @return true if address is a v4 mapped address
+  /**
+   * Returns true if address is a v4 mapped address
+   */
   bool isIPv4Mapped() const { return isV6() && asV6().isIPv4Mapped(); }
 
-  // @return true if address is uninitialized
+  /**
+   * Returns true if address is uninitialised
+   */
   bool empty() const { return family_ == AF_UNSPEC; }
 
-  // @return true if address is initialized
+  /**
+   * Returns true if address is initalised
+   */
   explicit operator bool() const { return !empty(); }
 
-  // @return true if this is an IPAddressV4 instance
+  /**
+   * Returns true if this represents an IPv4 address
+   */
   bool isV4() const { return family_ == AF_INET; }
 
-  // @return true if this is an IPAddressV6 instance
+  /**
+   * Returns true if this represents an IPv6 address
+   */
   bool isV6() const { return family_ == AF_INET6; }
 
-  // @return true if this address is all zeros
+  /**
+   * Returns true if the address is all zeros
+   */
   bool isZero() const {
     return pick([&](auto& _) { return _.isZero(); });
   }
 
-  // Number of bits in the address representation.
+  /**
+   * The number of bits in the address representation
+   */
   size_t bitCount() const {
     return pick([&](auto& _) { return _.bitCount(); });
   }
-  // Number of bytes in the address representation.
+
+  /**
+   * The number of bytes in the address representation
+   */
   size_t byteCount() const { return bitCount() / 8; }
-  // get nth most significant bit - 0 indexed
+
+  /**
+   * Get the nth most significant bit of the IP address (0-indexed).
+   * @param bitIndex n
+   */
   bool getNthMSBit(size_t bitIndex) const {
     return detail::getNthMSBitImpl(*this, bitIndex, family());
   }
-  // get nth most significant byte - 0 indexed
+
+  /**
+   * Get the nth most significant byte of the IP address (0-indexed).
+   * @param byteIndex n
+   */
   uint8_t getNthMSByte(size_t byteIndex) const;
-  // get nth bit - 0 indexed
+
+  /**
+   * Get the nth bit of the IP address (0-indexed).
+   * @param bitIndex n
+   */
   bool getNthLSBit(size_t bitIndex) const {
     return getNthMSBit(bitCount() - bitIndex - 1);
   }
-  // get nth byte - 0 indexed
+
+  /**
+   * Get the nth byte of the IP address (0-indexed).
+   * @param byteIndex n
+   */
   uint8_t getNthLSByte(size_t byteIndex) const {
     return getNthMSByte(byteCount() - byteIndex - 1);
   }
+
   /**
-   * Get human-readable string representation of the address.
+   * Get a json representation of the IP address.
    *
    * This prints a string representation of the address, for human consumption
    * or logging. The string will take the form of a JSON object that looks like:
-   * {family:'AF_INET|AF_INET6', addr:'address', hash:long}.
+   *  `{family:'AF_INET|AF_INET6', addr:'address', hash:long}`.
    */
   std::string toJson() const {
     return pick([&](auto& _) { return _.toJson(); });
   }
 
-  // Hash of address
+  /**
+   * Returns a hash of the IP address.
+   */
   std::size_t hash() const {
     return pick([&](auto& _) { return _.hash(); });
   }
 
-  // Return true if the address qualifies as localhost.
+  /**
+   * Return true if the IP address qualifies as localhost.
+   */
   bool isLoopback() const {
     return pick([&](auto& _) { return _.isLoopback(); });
   }
 
-  // Return true if the address qualifies as link local
+  /**
+   * Return true if the IP address qualifies as link local
+   */
   bool isLinkLocal() const {
     return pick([&](auto& _) { return _.isLinkLocal(); });
   }
 
-  // Return true if the address qualifies as broadcast.
+  /**
+   * Return true if the IP address qualifies as broadcast.
+   */
   bool isLinkLocalBroadcast() const {
     return pick([&](auto& _) { return _.isLinkLocalBroadcast(); });
   }
 
   /**
-   * Return true if the address is a special purpose address, as per rfc6890
-   * (i.e. 0.0.0.0).
+   * Return true if the IP address is a special purpose address, as defined per
+   * RFC 6890 (i.e. 0.0.0.0).
+   *
    * For V6, true if the address is not in one of global scope blocks:
    * 2000::/3, ffxe::/16.
    */
@@ -411,22 +493,28 @@ class IPAddress {
   }
 
   /**
-   * Return true if the address is private, as per rfc1918 and rfc4193
-   * (for example, 192.168.xxx.xxx or fc00::/7 addresses)
+   * Return true if the IP address is private, as per RFC 1918 and RFC 4193.
+   *
+   * For example, 192.168.xxx.xxx or fc00::/7 addresses.
    */
   bool isPrivate() const {
     return pick([&](auto& _) { return _.isPrivate(); });
   }
 
-  // Return true if the address is a multicast address.
+  /**
+   * Return true if the IP address is a multicast address.
+   */
   bool isMulticast() const {
     return pick([&](auto& _) { return _.isMulticast(); });
   }
 
   /**
-   * Creates IPAddress instance with all but most significant numBits set to 0.
+   * Creates an IPAddress instance with all but most significant numBits set to
+   * 0.
+   *
+   * @throws IPAddressFormatException if numBits > bitCount()
+   *
    * @param [in] numBits number of bits to mask
-   * @throws abort if numBits > bitCount()
    * @return IPAddress instance with bits set to 0
    */
   IPAddress mask(uint8_t numBits) const {
@@ -435,8 +523,10 @@ class IPAddress {
 
   /**
    * Provides a string representation of address.
+   *
+   * @throws IPAddressFormatException on `inet_ntop` error.
+   *
    * @note The string representation is calculated on demand.
-   * @throws IPAddressFormatException on inet_ntop error
    */
   std::string str() const {
     return pick([&](auto& _) { return _.str(); });
@@ -444,6 +534,7 @@ class IPAddress {
 
   /**
    * Return the fully qualified string representation of the address.
+   *
    * For V4 addresses this is the same as calling str(). For V6 addresses
    * this is the hex representation with : characters inserted every 4 digits.
    */
@@ -451,18 +542,22 @@ class IPAddress {
     return pick([&](auto& _) { return _.toFullyQualified(); });
   }
 
-  /// Same as toFullyQualified but append to an output string.
+  /**
+   * Same as toFullyQualified() but append to an output string.
+   */
   void toFullyQualifiedAppend(std::string& out) const {
     return pick([&](auto& _) { return _.toFullyQualifiedAppend(out); });
   }
 
-  // Address version (0 if empty, or 4 or 6 if nonempty)
+  /**
+   * Returns the IP address version. 0 if empty, 4 or 6 if nonempty.
+   */
   uint8_t version() const {
     return pick([&](auto& _) { return _.version(); });
   }
 
   /**
-   * Access to address bytes, in network byte order.
+   * Returns a pointer to the to IP address bytes, in network byte order.
    */
   const unsigned char* bytes() const {
     return pick([&](auto& _) { return _.bytes(); });
@@ -484,37 +579,74 @@ class IPAddress {
   sa_family_t family_;
 };
 
-// boost::hash uses hash_value() so this allows boost::hash to work
-// automatically for IPAddress
+/**
+ * `boost::hash` uses hash_value(), so this allows `boost::hash` to work
+ * automatically for IPAddress
+ */
 std::size_t hash_value(const IPAddress& addr);
+
+/**
+ * Appends a string representation of the IP address to the stream using str().
+ */
 std::ostream& operator<<(std::ostream& os, const IPAddress& addr);
-// Define toAppend() to allow IPAddress to be used with folly::to<string>
+
+/**
+ * @overloadbrief Define toAppend() to allow IPAddress to be used with
+ * `folly::to<string>`
+ */
 void toAppend(IPAddress addr, std::string* result);
 void toAppend(IPAddress addr, fbstring* result);
 
 /**
  * Return true if two addresses are equal.
  *
- * @note This takes into consideration V4 mapped addresses as well. If one
- *       address is v4 mapped we compare the v4 addresses.
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
  *
  * @return true if the two addresses are equal.
  */
 bool operator==(const IPAddress& addr1, const IPAddress& addr2);
-// Return true if addr1 < addr2
+
+/**
+ * Return true if `addr1 < addr2`
+ *
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
+ */
 bool operator<(const IPAddress& addr1, const IPAddress& addr2);
-// Derived operators
-inline bool operator!=(const IPAddress& a, const IPAddress& b) {
-  return !(a == b);
+
+/**
+ * Return true if two address are not equal
+ *
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
+ */
+inline bool operator!=(const IPAddress& addr1, const IPAddress& addr2) {
+  return !(addr1 == addr2);
 }
-inline bool operator>(const IPAddress& a, const IPAddress& b) {
-  return b < a;
+
+/**
+ * Return true if `addr1 > addr2`
+ *
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
+ */
+inline bool operator>(const IPAddress& addr1, const IPAddress& addr2) {
+  return addr2 < addr1;
 }
-inline bool operator<=(const IPAddress& a, const IPAddress& b) {
-  return !(a > b);
+
+/**
+ * Return true if `addr1 <= addr2`
+ *
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
+ */
+inline bool operator<=(const IPAddress& addr1, const IPAddress& addr2) {
+  return !(addr1 > addr2);
 }
-inline bool operator>=(const IPAddress& a, const IPAddress& b) {
-  return !(a < b);
+
+/**
+ * Return true if `addr1 >= addr2`
+ *
+ * V4-to-V6-mapped addresses are compared as V4 addresses.
+ */
+inline bool operator>=(const IPAddress& addr1, const IPAddress& addr2) {
+  return !(addr1 < addr2);
 }
 
 } // namespace folly
