@@ -61,6 +61,7 @@
 #include <type_traits>
 #include <utility>
 #if __cplusplus >= 201703L && __has_include(<optional>)
+#define FOLLY_HAS_STD_OPTIONAL
 #include <optional>
 #endif
 
@@ -98,6 +99,10 @@ class FOLLY_EXPORT OptionalEmptyException : public std::runtime_error {
       : std::runtime_error("Empty Optional cannot be unwrapped") {}
 };
 
+/**
+ * Optional is superseded by std::optional. Now that the C++ has a standardized
+ * implementation, Optional exists primarily for backward compatibility.
+ */
 template <class Value>
 class Optional {
  public:
@@ -112,6 +117,7 @@ class Optional {
       !std::is_abstract<Value>::value,
       "Optional may not be used with abstract types");
 
+  /// Default-constructed Optionals are None.
   constexpr Optional() noexcept {}
 
   Optional(const Optional& src) noexcept(
@@ -141,6 +147,13 @@ class Optional {
     construct(newValue);
   }
 
+  /**
+   * Creates an Optional with a value, where that value is constructed in-place.
+   *
+   * The in_place_t argument exists so that values can be default constructed
+   * (i.e. have no arguments), since this would otherwise be confused with
+   * default-constructing an Optional, which in turn results in None.
+   */
   template <typename... Args>
   constexpr explicit Optional(in_place_t, Args&&... args) noexcept(
       std::is_nothrow_constructible<Value, Args...>::value)
@@ -164,7 +177,8 @@ class Optional {
   }
 
 // Conversions to ease migration to std::optional
-#if __cplusplus >= 201703L && __has_include(<optional>)
+#ifdef FOLLY_HAS_STD_OPTIONAL
+  /// Allow construction of Optional from std::optional.
   template <
       typename U,
       typename = std::enable_if_t<std::is_same_v<U, std::optional<Value>>>>
@@ -184,6 +198,8 @@ class Optional {
       construct(*newValue);
     }
   }
+  /// Allow implict cast to std::optional
+  /// @methodset Migration
   explicit operator std::optional<Value>() && noexcept(
       std::is_nothrow_move_constructible<Value>::value) {
     std::optional<Value> ret = storage_.hasValue
@@ -199,6 +215,8 @@ class Optional {
   }
 #endif
 
+  /// Set the Optional
+  /// @methodset Modifiers
   void assign(const None&) { reset(); }
 
   void assign(Optional&& src) {
@@ -236,6 +254,7 @@ class Optional {
     }
   }
 
+  /// @methodset Modifiers
   Optional& operator=(None) noexcept {
     reset();
     return *this;
@@ -259,6 +278,8 @@ class Optional {
     return *this;
   }
 
+  /// Construct a new value in the Optional, in-place.
+  /// @methodset Modifiers
   template <class... Args>
   Value& emplace(Args&&... args) {
     reset();
@@ -276,10 +297,15 @@ class Optional {
     return value();
   }
 
+  /// Set the Optional to None
+  /// @methodset Modifiers
   void reset() noexcept { storage_.clear(); }
 
+  /// Set the Optional to None
+  /// @methodset Modifiers
   void clear() noexcept { reset(); }
 
+  /// @methodset Modifiers
   void swap(Optional& that) noexcept(IsNothrowSwappable<Value>::value) {
     if (hasValue() && that.hasValue()) {
       using std::swap;
@@ -293,6 +319,8 @@ class Optional {
     }
   }
 
+  /// Get the value. Must have value.
+  /// @methodset Getters
   constexpr const Value& value() const& {
     require_value();
     return storage_.value;
@@ -313,6 +341,8 @@ class Optional {
     return std::move(storage_.value);
   }
 
+  /// Get the value by pointer; nullptr if None.
+  /// @methodset Getters
   const Value* get_pointer() const& {
     return storage_.hasValue ? &storage_.value : nullptr;
   }
@@ -321,21 +351,32 @@ class Optional {
   }
   Value* get_pointer() && = delete;
 
+  /// Does this Optional have a value.
+  /// @methodset Observers
   constexpr bool has_value() const noexcept { return storage_.hasValue; }
 
+  /// Does this Optional have a value.
+  /// @methodset Observers
   constexpr bool hasValue() const noexcept { return has_value(); }
 
+  /// Does this Optional have a value.
+  /// @methodset Observers
   constexpr explicit operator bool() const noexcept { return has_value(); }
 
+  /// Get the value. Must have value.
+  /// @methodset Getters
   constexpr const Value& operator*() const& { return value(); }
   constexpr Value& operator*() & { return value(); }
   constexpr const Value&& operator*() const&& { return std::move(value()); }
   constexpr Value&& operator*() && { return std::move(value()); }
 
+  /// Get the value. Must have value.
+  /// @methodset Getters
   constexpr const Value* operator->() const { return &value(); }
   constexpr Value* operator->() { return &value(); }
 
-  // Return a copy of the value if set, or a given default if not.
+  /// Return a copy of the value if set, or a given default if not.
+  /// @methodset Getters
   template <class U>
   constexpr Value value_or(U&& dflt) const& {
     if (storage_.hasValue) {
