@@ -177,52 +177,6 @@ constexpr bool kIsSanitize = false;
 #define FOLLY_PACK_POP /**/
 #endif
 
-// Generalize warning push/pop.
-#if defined(__GNUC__) || defined(__clang__)
-// Clang & GCC
-#define FOLLY_PUSH_WARNING _Pragma("GCC diagnostic push")
-#define FOLLY_POP_WARNING _Pragma("GCC diagnostic pop")
-#define FOLLY_GNU_DISABLE_WARNING_INTERNAL2(warningName) #warningName
-#define FOLLY_GNU_DISABLE_WARNING(warningName) \
-  _Pragma(                                     \
-      FOLLY_GNU_DISABLE_WARNING_INTERNAL2(GCC diagnostic ignored warningName))
-#ifdef __clang__
-#define FOLLY_CLANG_DISABLE_WARNING(warningName) \
-  FOLLY_GNU_DISABLE_WARNING(warningName)
-#define FOLLY_GCC_DISABLE_WARNING(warningName)
-#else
-#define FOLLY_CLANG_DISABLE_WARNING(warningName)
-#define FOLLY_GCC_DISABLE_WARNING(warningName) \
-  FOLLY_GNU_DISABLE_WARNING(warningName)
-#endif
-#define FOLLY_MSVC_DISABLE_WARNING(warningNumber)
-#elif defined(_MSC_VER)
-#define FOLLY_PUSH_WARNING __pragma(warning(push))
-#define FOLLY_POP_WARNING __pragma(warning(pop))
-// Disable the GCC warnings.
-#define FOLLY_GNU_DISABLE_WARNING(warningName)
-#define FOLLY_GCC_DISABLE_WARNING(warningName)
-#define FOLLY_CLANG_DISABLE_WARNING(warningName)
-#define FOLLY_MSVC_DISABLE_WARNING(warningNumber) \
-  __pragma(warning(disable : warningNumber))
-#else
-#define FOLLY_PUSH_WARNING
-#define FOLLY_POP_WARNING
-#define FOLLY_GNU_DISABLE_WARNING(warningName)
-#define FOLLY_GCC_DISABLE_WARNING(warningName)
-#define FOLLY_CLANG_DISABLE_WARNING(warningName)
-#define FOLLY_MSVC_DISABLE_WARNING(warningNumber)
-#endif
-
-#ifdef FOLLY_HAVE_SHADOW_LOCAL_WARNINGS
-#define FOLLY_GCC_DISABLE_NEW_SHADOW_WARNINGS            \
-  FOLLY_GNU_DISABLE_WARNING("-Wshadow-compatible-local") \
-  FOLLY_GNU_DISABLE_WARNING("-Wshadow-local")            \
-  FOLLY_GNU_DISABLE_WARNING("-Wshadow")
-#else
-#define FOLLY_GCC_DISABLE_NEW_SHADOW_WARNINGS /* empty */
-#endif
-
 // It turns out that GNU libstdc++ and LLVM libc++ differ on how they implement
 // the 'std' namespace; the latter uses inline namespaces. Wrap this decision
 // up in a macro to make forward-declarations easier.
@@ -250,9 +204,6 @@ constexpr bool kIsSanitize = false;
 #ifdef _MSC_VER
 #include <folly/portability/SysTypes.h>
 
-// Hide a GCC specific thing that breaks MSVC if left alone.
-#define __extension__
-
 // We have compiler support for the newest of the new, but
 // MSVC doesn't tell us that.
 //
@@ -264,6 +215,9 @@ constexpr bool kIsSanitize = false;
 #define __SSE4_2__ 1
 #endif // !defined(_M_ARM) && !defined(_M_ARM64)
 
+// Hide a GCC specific thing that breaks MSVC if left alone.
+#define __extension__
+
 // compiler specific to compiler specific
 // nolint
 #define __PRETTY_FUNCTION__ __FUNCSIG__
@@ -377,13 +331,6 @@ constexpr auto kHasWeakSymbols = false;
 #endif
 #endif
 
-#if FOLLY_UNUSUAL_GFLAGS_NAMESPACE
-namespace FOLLY_GFLAGS_NAMESPACE {}
-namespace gflags {
-using namespace FOLLY_GFLAGS_NAMESPACE;
-} // namespace gflags
-#endif
-
 // RTTI may not be enabled for this compilation unit.
 #if defined(__GXX_RTTI) || defined(__cpp_rtti) || \
     (defined(_MSC_VER) && defined(_CPPRTTI))
@@ -502,7 +449,7 @@ constexpr auto kMscVer = _MSC_VER;
 constexpr auto kMscVer = 0;
 #endif
 
-#if __GNUC__
+#if defined(__GNUC__) && __GNUC__
 constexpr auto kGnuc = __GNUC__;
 #else
 constexpr auto kGnuc = 0;
@@ -594,13 +541,15 @@ constexpr auto kCpplibVer = 0;
 // <experimental/coroutine> which will conflict with anyone who wants to load
 // the LLVM implementation of coroutines on Windows.
 #define FOLLY_HAS_COROUTINES 0
-#elif (__cpp_coroutines >= 201703L || __cpp_impl_coroutine >= 201902L) && \
+#elif (                                                                    \
+    (defined(__cpp_coroutines) && __cpp_coroutines >= 201703L) ||          \
+    (defined(__cpp_impl_coroutine) && __cpp_impl_coroutine >= 201902L)) && \
     (__has_include(<coroutine>) || __has_include(<experimental/coroutine>))
 #define FOLLY_HAS_COROUTINES 1
 // This is mainly to workaround bugs triggered by LTO, when stack allocated
 // variables in await_suspend end up on a coroutine frame.
 #define FOLLY_CORO_AWAIT_SUSPEND_NONTRIVIAL_ATTRIBUTES FOLLY_NOINLINE
-#elif _MSC_VER && _RESUMABLE_FUNCTIONS_SUPPORTED
+#elif defined(_MSC_VER) && _MSC_VER && _RESUMABLE_FUNCTIONS_SUPPORTED
 // NOTE: MSVC 2017 does not currently support the full Coroutines TS since it
 // does not yet support symmetric-transfer.
 #define FOLLY_HAS_COROUTINES 0
@@ -618,7 +567,7 @@ constexpr auto kCpplibVer = 0;
 #define FOLLY_HAVE_NOEXCEPT_FUNCTION_TYPE 1
 #endif
 
-#if __cpp_inline_variables >= 201606L
+#if __cpp_inline_variables >= 201606L || FOLLY_CPLUSPLUS >= 201703L
 #define FOLLY_HAS_INLINE_VARIABLES 1
 #define FOLLY_INLINE_VARIABLE inline
 #else
