@@ -23,10 +23,12 @@
 #endif
 
 #include <atomic>
+#include <chrono>
 #include <cstdio>
 #include <cstdlib>
 #include <iostream>
 #include <string>
+#include <fmt/chrono.h>
 
 #include <folly/Demangle.h>
 #include <folly/ScopeGuard.h>
@@ -282,7 +284,7 @@ void SingletonVault::addEagerInitOnReenableSingleton(
 }
 
 void SingletonVault::registrationComplete() {
-  std::atexit([]() { SingletonVault::singleton()->destroyInstances(); });
+  scheduleDestroyInstances();
 
   auto state = state_.wlock();
   state->check(detail::SingletonVaultState::Type::Running);
@@ -358,6 +360,8 @@ void SingletonVault::doEagerInitVia(Executor& exe, folly::Baton<>* done) {
 }
 
 void SingletonVault::destroyInstances() {
+  cancellationSource_.wlock()->requestCancellation();
+
   auto stateW = state_.wlock();
   if (stateW->state == detail::SingletonVaultState::Type::Quiescing) {
     return;
@@ -402,6 +406,8 @@ void SingletonVault::destroyInstances() {
 }
 
 void SingletonVault::reenableInstances() {
+  CHECK(!shutdownTimerStarted_.load(std::memory_order_relaxed))
+      << "reenableInstances() called after destroyInstancesFinal()";
   {
     auto state = state_.wlock();
 
@@ -410,6 +416,11 @@ void SingletonVault::reenableInstances() {
     state->state = detail::SingletonVaultState::Type::Running;
   }
 
+  // reset the cancellation source
+  cancellationSource_.withWLock([&](auto& cancellationSource) {
+    cancellationSource = folly::CancellationSource{};
+  });
+
   auto eagerInitOnReenableSingletons = eagerInitOnReenableSingletons_.copy();
   auto instantiatedAtLeastOnce = instantiatedAtLeastOnce_.copy();
   for (auto* single : eagerInitOnReenableSingletons) {
@@ -424,14 +435,21 @@ void SingletonVault::scheduleDestroyInstances() {
   // Add a dependency on folly::ThreadLocal to make sure all its static
   // singletons are initalized first.
   threadlocal_detail::StaticMeta<void, void>::instance();
-  std::atexit([] {
-    SingletonVault::singleton()->startShutdownTimer();
-    SingletonVault::singleton()->destroyInstances();
-  });
+  std::atexit([] { SingletonVault::singleton()->destroyInstancesFinal(); });
+}
+
+void SingletonVault::destroyInstancesFinal() {
+  startShutdownTimer();
+  destroyInstances();
 }
 
 void SingletonVault::addToShutdownLog(std::string message) {
-  shutdownLog_.wlock()->push_back(std::move(message));
+  std::chrono::time_point<std::chrono::system_clock> now =
+      std::chrono::system_clock::now();
+  std::chrono::milliseconds millis =
+      std::chrono::duration_cast<std::chrono::milliseconds>(
+          now.time_since_epoch());
+  shutdownLog_.wlock()->push_back(fmt::format("{:%T} {}", millis, message));
 }
 
 #if FOLLY_HAVE_LIBRT
