@@ -95,6 +95,8 @@ class RandomDataHolder : public DataHolder {
 };
 
 RandomDataHolder::RandomDataHolder(size_t sizeLog2) : DataHolder(sizeLog2) {
+  memset(data_.get(), 0, size_);
+
   static constexpr size_t numThreadsLog2 = 3;
   static constexpr size_t numThreads = size_t(1) << numThreadsLog2;
 
@@ -334,9 +336,9 @@ void CompressionVarintTest::runSimpleTest(const DataHolder& dh) {
   auto original = IOBuf::wrapBuffer(dh.data(uncompressedLength_));
   auto compressed = codec_->compress(original.get());
   auto breakPoint =
-      1UL +
+      1ULL +
       Random::rand64(
-          std::max(uint64_t(9), oneBasedMsbPos(uncompressedLength_)) / 9UL);
+          std::max(uint64_t(9), oneBasedMsbPos(uncompressedLength_)) / 9ULL);
   auto tinyBuf = IOBuf::copyBuffer(
       compressed->data(), std::min<size_t>(compressed->length(), breakPoint));
   compressed->trimStart(breakPoint);
