@@ -137,6 +137,10 @@ namespace folly {
 ///   a fixed size of 2^LgSegmentSize entries. Each segment is used
 ///   exactly once.
 /// - Each entry is composed of a futex and a single element.
+/// - Each segment's array of entries is strided to avoid false sharing.
+///   I.e., to reduce any cacheline contention that might be induced by
+///   concurrent mutations to the queue that might happen to affect
+///   otherwise-adjacent locations that might happen to share cacheline.
 /// - The queue contains two 64-bit ticket variables. The producer
 ///   ticket counts the number of producer tickets issued so far, and
 ///   the same for the consumer ticket. Each ticket number corresponds
@@ -332,8 +336,7 @@ class UnboundedQueue {
 
   /** try_peek */
   FOLLY_ALWAYS_INLINE const T* try_peek() noexcept {
-    /* This function is supported only for USPSC and UMPSC queues. */
-    DCHECK(SingleConsumer);
+    static_assert(SingleConsumer, "not single-consumer");
     return tryPeekUntil(std::chrono::steady_clock::time_point::min());
   }
 
@@ -499,6 +502,8 @@ class UnboundedQueue {
   template <typename Clock, typename Duration>
   FOLLY_ALWAYS_INLINE const T* tryPeekUntil(
       const std::chrono::time_point<Clock, Duration>& deadline) noexcept {
+    // This function is supported only for USPSC and UMPSC queues.
+    DCHECK(SingleConsumer);
     Segment* s = head();
     Ticket t = consumerTicket();
     DCHECK_GE(t, s->minTicket());
