@@ -1142,30 +1142,33 @@ TEST(AccessSpreader, Wrapping) {
   }
 }
 
-TEST(CoreRawAllocator, Basic) {
-  CoreRawAllocator<32> alloc;
-  auto& a = alloc.get(0);
-  auto res = a.allocate(8);
+TEST(CoreAllocator, Basic) {
+  constexpr size_t kNumStripes = 32;
+
+  auto res = coreMalloc(8, kNumStripes, 0);
   memset(res, 0, 8);
-  a.deallocate(res);
-  res = a.allocate(8);
+  coreFree(res);
+
+  res = coreMalloc(8, kNumStripes, 0);
   EXPECT_TRUE((intptr_t)res % 8 == 0); // check alignment
   memset(res, 0, 8);
-  a.deallocate(res);
-  res = a.allocate(12);
+  coreFree(res);
+  res = coreMalloc(12, kNumStripes, 0);
   EXPECT_TRUE((intptr_t)res % 16 == 0); // check alignment
   memset(res, 0, 12);
-  a.deallocate(res);
-  res = a.allocate(257);
+  coreFree(res);
+  res = coreMalloc(257, kNumStripes, 0);
   memset(res, 0, 257);
-  a.deallocate(res);
+  coreFree(res);
 
-  std::vector<void*> mems;
+  CoreAllocator<int> a;
+  std::vector<int*> mems;
   for (int i = 0; i < 10000; i++) {
+    CoreAllocatorGuard g(kNumStripes, i % kNumStripes);
     mems.push_back(a.allocate(1));
   }
   for (auto& mem : mems) {
-    a.deallocate(mem);
+    a.deallocate(mem, 1);
   }
   mems.clear();
 }
