@@ -15,7 +15,7 @@
  */
 
 /**
- * dyanmic is a runtime dynamically typed value.  It holds types from a specific
+ * dynamic is a runtime dynamically typed value.  It holds types from a specific
  * predetermined set of types: int, double, bool, nullptr_t, string, array,
  * map. In particular, it can be used as a convenient in-memory representation
  * for complete JSON objects.
@@ -31,7 +31,7 @@
  * https://github.com/facebook/folly/blob/main/folly/docs/Dynamic.md
  *
  * @author Jordan DeLong <delong.j@fb.com>
- * @refcode docs/examples/folly/dynamic.cpp
+ * @refcode folly/docs/examples/folly/dynamic.cpp
  * @struct folly::dynamic
  */
 /*
@@ -175,7 +175,7 @@ struct dynamic {
    * See D3013423 and
    * [DR95](http://www.open-std.org/jtc1/sc22/wg21/docs/cwg_defects.html#1467).
    *
-   * @refcode docs/examples/folly/dynamic/array.cpp
+   * @refcode folly/docs/examples/folly/dynamic/array.cpp
    * @methodset Array
    */
   template <class... Args>
@@ -184,7 +184,7 @@ struct dynamic {
   /**
    * Construct a dynamic object.
    *
-   * @refcode docs/examples/folly/dynamic/object.cpp
+   * @refcode folly/docs/examples/folly/dynamic/object.cpp
    * @methodset Object
    */
   static ObjectMaker object();
@@ -1076,6 +1076,7 @@ struct dynamic {
   T asImpl() const;
 
   static char const* typeName(Type);
+  // NOTE: like ~dynamic, destroy() leaves type_ and u_ in an invalid state.
   void destroy() noexcept;
   void print(std::ostream&) const;
   void print_as_pseudo_json(std::ostream&) const; // see json.cpp
@@ -1273,4 +1274,15 @@ auto make_dynamic_view(dynamic&&) = delete;
 
 } // namespace folly
 
+namespace std {
+
+template <>
+struct hash<::folly::dynamic> {
+  using folly_is_avalanching = std::true_type;
+
+  size_t operator()(::folly::dynamic const& d) const { return d.hash(); }
+};
+
+} // namespace std
+
 #include <folly/dynamic-inl.h>
