@@ -145,7 +145,7 @@ class ThreadPoolExecutor : public DefaultKeepAliveExecutor {
     uint64_t requestId;
   };
 
-  using TaskStatsCallback = std::function<void(TaskStats)>;
+  using TaskStatsCallback = std::function<void(const TaskStats&)>;
   void subscribeToTaskStats(TaskStatsCallback cb);
 
   /**
@@ -214,15 +214,20 @@ class ThreadPoolExecutor : public DefaultKeepAliveExecutor {
   typedef std::shared_ptr<Thread> ThreadPtr;
 
   struct Task {
-    explicit Task(
+    struct Expiration {
+      std::chrono::milliseconds expiration;
+      Func expireCallback;
+    };
+
+    Task(
         Func&& func,
         std::chrono::milliseconds expiration,
         Func&& expireCallback);
+
     Func func_;
     std::chrono::steady_clock::time_point enqueueTime_;
-    std::chrono::milliseconds expiration_;
-    Func expireCallback_;
     std::shared_ptr<folly::RequestContext> context_;
+    std::unique_ptr<Expiration> expiration_;
   };
 
   void runTask(const ThreadPtr& thread, Task&& task);
@@ -325,6 +330,7 @@ class ThreadPoolExecutor : public DefaultKeepAliveExecutor {
   folly::ThreadPoolListHook threadPoolHook_;
 
   // Dynamic thread sizing functions and variables
+  void ensureMaxActiveThreads();
   void ensureActiveThreads();
   void ensureJoined();
   bool minActive();
@@ -337,7 +343,7 @@ class ThreadPoolExecutor : public DefaultKeepAliveExecutor {
   std::atomic<size_t> activeThreads_{0};
 
   std::atomic<size_t> threadsToJoin_{0};
-  std::chrono::milliseconds threadTimeout_{0};
+  std::atomic<std::chrono::milliseconds> threadTimeout_;
 
   void joinKeepAliveOnce() {
     if (!std::exchange(keepAliveJoined_, true)) {
