@@ -39,10 +39,6 @@
 #include <folly/lang/Assume.h>
 #include <folly/lang/Bits.h>
 
-#if !FOLLY_X64
-#error EliasFanoCoding.h requires x86_64
-#endif
-
 namespace folly {
 namespace compression {
 
@@ -683,7 +679,10 @@ class EliasFanoReader {
   using SizeType = SizeT;
 
   explicit EliasFanoReader(const typename Encoder::CompressedList& list)
-      : upper_(list), lower_(list.lower), numLowerBits_(list.numLowerBits) {
+      : upper_(list),
+        lower_(list.lower),
+        value_(),
+        numLowerBits_(list.numLowerBits) {
     DCHECK_LE(list.size, std::numeric_limits<SizeType>::max());
     DCHECK(Instructions::supported());
   }
