@@ -69,7 +69,7 @@ namespace channels {
  *  Receiver<std::string> outputReceiver = transform(
  *      getInputReceiver(),
  *      getExecutor(),
- *      [](folly::Try<int> try) -> folly::coro::AsyncGenerator<std::string&&> {
+ *      [](Try<int> try) -> folly::coro::AsyncGenerator<std::string&&> {
  *          co_yield folly::to<std::string>(try.value());
  *      });
  */
@@ -77,9 +77,9 @@ template <
     typename ReceiverType,
     typename TransformValueFunc,
     typename InputValueType = typename ReceiverType::ValueType,
-    typename OutputValueType = typename folly::invoke_result_t<
+    typename OutputValueType = typename folly::invoke_result_t< //
         TransformValueFunc,
-        folly::Try<InputValueType>>::value_type>
+        Try<InputValueType>>::value_type>
 Receiver<OutputValueType> transform(
     ReceiverType inputReceiver,
     folly::Executor::KeepAlive<folly::SequencedExecutor> executor,
@@ -93,7 +93,7 @@ Receiver<OutputValueType> transform(
  * folly::Executor::KeepAlive<folly::SequencedExecutor> getExecutor();
  *
  * folly::coro::AsyncGenerator<OutputValueType&&> transformValue(
- *     folly::Try<InputValueType> inputValue);
+ *     Try<InputValueType> inputValue);
  *
  * std::shared_ptr<RateLimiter> getRateLimiter(); // Can return nullptr
  */
@@ -103,7 +103,7 @@ template <
     typename InputValueType = typename ReceiverType::ValueType,
     typename OutputValueType =
         typename decltype(std::declval<TransformerType>().transformValue(
-            std::declval<folly::Try<InputValueType>>()))::value_type>
+            std::declval<Try<InputValueType>>()))::value_type>
 Receiver<OutputValueType> transform(
     ReceiverType inputReceiver, TransformerType transformer);
 
@@ -162,7 +162,7 @@ Receiver<OutputValueType> transform(
  *              std::vector<std::string>({"Initialized"}),
  *              getInputReceiver(initializeArg));
  *      },
- *      [](folly::Try<int> try) -> folly::coro::AsyncGenerator<std::string&&> {
+ *      [](Try<int> try) -> folly::coro::AsyncGenerator<std::string&&> {
  *          try {
  *            co_yield folly::to<std::string>(try.value());
  *          } catch (const SomeApplicationException& ex) {
@@ -179,9 +179,9 @@ template <
         InitializeTransformFunc,
         InitializeArg>::StorageType::second_type,
     typename InputValueType = typename ReceiverType::ValueType,
-    typename OutputValueType = typename folly::invoke_result_t<
+    typename OutputValueType = typename folly::invoke_result_t< //
         TransformValueFunc,
-        folly::Try<InputValueType>>::value_type>
+        Try<InputValueType>>::value_type>
 Receiver<OutputValueType> resumableTransform(
     folly::Executor::KeepAlive<folly::SequencedExecutor> executor,
     InitializeArg initializeArg,
@@ -199,7 +199,7 @@ Receiver<OutputValueType> resumableTransform(
  * initializeTransform(InitializeArg initializeArg);
  *
  * folly::coro::AsyncGenerator<OutputValueType&&> transformValue(
- *     folly::Try<InputValueType> inputValue);
+ *     Try<InputValueType> inputValue);
  *
  * std::shared_ptr<RateLimiter> getRateLimiter(); // Can return nullptr
  */
@@ -212,7 +212,7 @@ template <
     typename InputValueType = typename ReceiverType::ValueType,
     typename OutputValueType =
         typename decltype(std::declval<TransformerType>().transformValue(
-            std::declval<folly::Try<InputValueType>>()))::value_type>
+            std::declval<Try<InputValueType>>()))::value_type>
 Receiver<OutputValueType> resumableTransform(
     InitializeArg initializeArg, TransformerType transformer);
 
