@@ -31,8 +31,10 @@
 #include <folly/CPortability.h>
 #include <folly/Conv.h>
 #include <folly/CppAttributes.h>
+#include <folly/ExceptionString.h>
 #include <folly/Function.h>
 #include <folly/Range.h>
+#include <folly/experimental/io/IoUringBase.h>
 #include <folly/io/IOBuf.h>
 #include <folly/io/async/EventBaseBackendBase.h>
 #include <folly/portability/Asm.h>
@@ -168,6 +170,12 @@ class IoUringBackend : public EventBaseBackendBase {
       return *this;
     }
 
+    Options& setDeferTaskRun(bool v) {
+      deferTaskRun = v;
+
+      return *this;
+    }
+
     size_t capacity{256};
     size_t minCapacity{0};
     size_t maxSubmit{128};
@@ -177,6 +185,7 @@ class IoUringBackend : public EventBaseBackendBase {
     bool registerRingFd{false};
     uint32_t flags{0};
     bool taskRunCoop{false};
+    bool deferTaskRun{false};
 
     std::chrono::milliseconds sqIdle{0};
     std::chrono::milliseconds cqIdle{0};
@@ -187,69 +196,19 @@ class IoUringBackend : public EventBaseBackendBase {
     size_t initalProvidedBuffersEachSize{0};
   };
 
-  struct IoSqeBase
-      : boost::intrusive::list_base_hook<
-            boost::intrusive::link_mode<boost::intrusive::auto_unlink>> {
-    IoSqeBase() = default;
-    // use raw addresses, so disallow copy/move
-    IoSqeBase(IoSqeBase&&) = delete;
-    IoSqeBase(const IoSqeBase&) = delete;
-    IoSqeBase& operator=(IoSqeBase&&) = delete;
-    IoSqeBase& operator=(const IoSqeBase&) = delete;
-
-    virtual ~IoSqeBase() = default;
-    virtual void processSubmit(struct io_uring_sqe* sqe) = 0;
-    virtual void callback(int res, uint32_t flags) = 0;
-    virtual void callbackCancelled() = 0;
-    bool inFlight() const { return inFlight_; }
-    bool cancelled() const { return cancelled_; }
-    void markCancelled() { cancelled_ = true; }
-
-   private:
-    friend class IoUringBackend;
-    void internalSubmit(struct io_uring_sqe* sqe);
-    void internalCallback(int res, uint32_t flags);
-    void internalUnmarkInflight();
-
-    bool inFlight_ = false;
-    bool cancelled_ = false;
-  };
-
-  class ProvidedBufferProviderBase {
-   protected:
-    uint16_t const gid_;
-    int const count_;
-    size_t const sizePerBuffer_;
-
-   public:
-    explicit ProvidedBufferProviderBase(
-        uint16_t gid, uint32_t count, size_t sizePerBuffer)
-        : gid_(gid), count_(count), sizePerBuffer_(sizePerBuffer) {}
-    virtual ~ProvidedBufferProviderBase() = default;
-
-    ProvidedBufferProviderBase(ProvidedBufferProviderBase&&) = delete;
-    ProvidedBufferProviderBase(ProvidedBufferProviderBase const&) = delete;
-    ProvidedBufferProviderBase& operator=(ProvidedBufferProviderBase&&) =
-        delete;
-    ProvidedBufferProviderBase& operator=(ProvidedBufferProviderBase const&) =
-        delete;
-
-    size_t sizePerBuffer() const { return sizePerBuffer_; }
-    uint16_t gid() const { return gid_; }
-
-    virtual uint32_t count() const = 0;
-    virtual void unusedBuf(uint16_t i, size_t length) = 0;
-    virtual std::unique_ptr<IOBuf> getIoBuf(uint16_t i, size_t length) = 0;
-    virtual void enobuf() = 0;
-    virtual bool available() const = 0;
-  };
-
   explicit IoUringBackend(Options options);
   ~IoUringBackend() override;
+  Options const& options() const { return options_; }
 
+  bool isWaitingToSubmit() const {
+    return waitingToSubmit_ || !submitList_.empty();
+  }
   struct io_uring* ioRingPtr() {
     return &ioRing_;
   }
+  struct io_uring_params const& params() const {
+    return params_;
+  }
 
   // from EventBaseBackendBase
   event_base* getEventBase() override { return nullptr; }
@@ -262,22 +221,25 @@ class IoUringBackend : public EventBaseBackendBase {
 
   bool eb_event_active(Event&, int) override { return false; }
 
+  size_t loopPoll();
+  void submitOutstanding();
+  unsigned int processCompleted();
+
   // returns true if the current Linux kernel version
   // supports the io_uring backend
   static bool isAvailable();
   bool kernelHasNonBlockWriteFixes() const;
   static bool kernelSupportsRecvmsgMultishot();
+  static bool kernelSupportsDeferTaskrun();
+  static bool kernelSupportsSendZC();
 
-  struct FdRegistrationRecord : public boost::intrusive::slist_base_hook<
-                                    boost::intrusive::cache_last<false>> {
-    int count_{0};
-    int fd_{-1};
-    size_t idx_{0};
-  };
+  IoUringFdRegistrationRecord* registerFd(int fd) noexcept {
+    return fdRegistry_.alloc(fd);
+  }
 
-  FdRegistrationRecord* registerFd(int fd) { return fdRegistry_.alloc(fd); }
-
-  bool unregisterFd(FdRegistrationRecord* rec) { return fdRegistry_.free(rec); }
+  bool unregisterFd(IoUringFdRegistrationRecord* rec) {
+    return fdRegistry_.free(rec);
+  }
 
   // CQ poll mode loop callback
   using CQPollLoopCallback = folly::Function<void()>;
@@ -351,13 +313,14 @@ class IoUringBackend : public EventBaseBackendBase {
     submitImmediateIoSqe(ioSqe);
   }
 
-  void submitSoon(IoSqeBase& ioSqe);
+  void submitNextLoop(IoSqeBase& ioSqe) noexcept;
+  void submitSoon(IoSqeBase& ioSqe) noexcept;
   void submitNow(IoSqeBase& ioSqe);
   void submitNowNoCqe(IoSqeBase& ioSqe, int count = 1);
   void cancel(IoSqeBase* sqe);
 
   // built in buffer provider
-  ProvidedBufferProviderBase* bufferProvider() { return bufferProvider_.get(); }
+  IoUringBufferProviderBase* bufferProvider() { return bufferProvider_.get(); }
   uint16_t nextBufferProviderGid() { return bufferProviderGidNext_++; }
 
  protected:
@@ -381,17 +344,10 @@ class IoUringBackend : public EventBaseBackendBase {
   };
 
   struct UserData {
-    uint64_t value;
-    explicit UserData(uint64_t i) noexcept : value{i} {}
-    template <
-        typename T = int,
-        std::enable_if_t<sizeof(void*) == sizeof(uint64_t), T> = 0>
+    uintptr_t value;
     explicit UserData(void* p) noexcept
         : value{reinterpret_cast<uintptr_t>(p)} {}
     /* implicit */ operator uint64_t() const noexcept { return value; }
-    template <
-        typename T = int,
-        std::enable_if_t<sizeof(void*) == sizeof(uint64_t), T> = 0>
     /* implicit */ operator void*() const noexcept {
       return reinterpret_cast<void*>(value);
     }
@@ -443,16 +399,16 @@ class IoUringBackend : public EventBaseBackendBase {
   struct IoSqe;
 
   static void processPollIoSqe(
-      IoUringBackend* backend, IoSqe* ioSqe, int64_t res, uint32_t flags);
+      IoUringBackend* backend, IoSqe* ioSqe, int res, uint32_t flags);
   static void processTimerIoSqe(
       IoUringBackend* backend,
       IoSqe* /*sqe*/,
-      int64_t /*res*/,
+      int /*res*/,
       uint32_t /* flags */);
   static void processSignalReadIoSqe(
       IoUringBackend* backend,
       IoSqe* /*sqe*/,
-      int64_t /*res*/,
+      int /*res*/,
       uint32_t /* flags */);
 
   // signal handling
@@ -462,27 +418,32 @@ class IoUringBackend : public EventBaseBackendBase {
   size_t processSignals();
   void setProcessSignals();
 
-  void processPollIo(IoSqe* ioSqe, int64_t res, uint32_t flags) noexcept;
+  void processPollIo(IoSqe* ioSqe, int res, uint32_t flags) noexcept;
 
   IoSqe* FOLLY_NULLABLE allocIoSqe(const EventCallback& cb);
-  void releaseIoSqe(IoSqe* aioIoSqe);
-  void incNumIoSqeInUse() { numIoSqeInUse_++; }
+  void releaseIoSqe(IoSqe* aioIoSqe) noexcept;
 
   // submit immediate if POLL_SQ | POLL_SQ_IMMEDIATE_IO flags are set
   void submitImmediateIoSqe(IoSqeBase& ioSqe);
 
-  void internalSubmit(IoSqeBase& ioSqe);
+  void internalSubmit(IoSqeBase& ioSqe) noexcept;
+
+  enum class InternalProcessCqeMode {
+    NORMAL, // process existing and any available
+    AVAILABLE_ONLY, // process existing but don't get more
+    CANCEL_ALL, // cancel every sqe
+  };
+  unsigned int internalProcessCqe(
+      unsigned int maxGet, InternalProcessCqeMode mode) noexcept;
 
   int eb_event_modify_inserted(Event& event, IoSqe* ioSqe);
 
-  FOLLY_ALWAYS_INLINE size_t numIoSqeInUse() const { return numIoSqeInUse_; }
-
   struct FdRegistry {
     FdRegistry() = delete;
     FdRegistry(struct io_uring& ioRing, size_t n);
 
-    FdRegistrationRecord* alloc(int fd);
-    bool free(FdRegistrationRecord* record);
+    IoUringFdRegistrationRecord* alloc(int fd) noexcept;
+    bool free(IoUringFdRegistrationRecord* record);
 
     int init();
     size_t update();
@@ -491,35 +452,35 @@ class IoUringBackend : public EventBaseBackendBase {
     struct io_uring& ioRing_;
     std::vector<int> files_;
     size_t inUse_;
-    std::vector<FdRegistrationRecord> records_;
+    std::vector<IoUringFdRegistrationRecord> records_;
     boost::intrusive::
-        slist<FdRegistrationRecord, boost::intrusive::cache_last<false>>
+        slist<IoUringFdRegistrationRecord, boost::intrusive::cache_last<false>>
             free_;
   };
 
   struct IoSqe : public IoSqeBase {
-    using BackendCb = void(IoUringBackend*, IoSqe*, int64_t, uint32_t);
+    using BackendCb = void(IoUringBackend*, IoSqe*, int, uint32_t);
     explicit IoSqe(
         IoUringBackend* backend = nullptr,
         bool poolAlloc = false,
         bool persist = false)
         : backend_(backend), poolAlloc_(poolAlloc), persist_(persist) {}
-    virtual ~IoSqe() = default;
+    virtual ~IoSqe() override = default;
 
-    void callback(int res, uint32_t flags) override {
+    void callback(int res, uint32_t flags) noexcept override {
       backendCb_(backend_, this, res, flags);
     }
-    void callbackCancelled() override { release(); }
-    virtual void release();
+    void callbackCancelled(int, uint32_t) noexcept override { release(); }
+    virtual void release() noexcept;
 
     IoUringBackend* backend_;
     BackendCb* backendCb_{nullptr};
     const bool poolAlloc_;
     const bool persist_;
     Event* event_{nullptr};
-    FdRegistrationRecord* fdRecord_{nullptr};
+    IoUringFdRegistrationRecord* fdRecord_{nullptr};
     size_t useCount_{0};
-    int64_t res_;
+    int res_;
     uint32_t cqeFlags_;
 
     FOLLY_ALWAYS_INLINE void resetEvent() {
@@ -531,7 +492,7 @@ class IoUringBackend : public EventBaseBackendBase {
       }
     }
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       auto* ev = event_->getEvent();
       if (ev) {
         const auto& cb = event_->getCallback();
@@ -564,22 +525,13 @@ class IoUringBackend : public EventBaseBackendBase {
           case EventCallback::Type::TYPE_RECVMSG_MULTISHOT:
             if (auto* hdr =
                     cb.recvmsgMultishotCb_->allocateRecvmsgMultishotData()) {
-              prepRecvmsgMultishot(
-                  sqe,
-                  ev->ev_fd,
-                  &hdr->data_,
-                  (ev->ev_events & EV_PERSIST) != 0);
+              prepRecvmsgMultishot(sqe, ev->ev_fd, &hdr->data_);
               cbData_.set(hdr);
               return;
             }
             break;
         }
-
-        prepPollAdd(
-            sqe,
-            ev->ev_fd,
-            getPollFlags(ev->ev_events),
-            (ev->ev_events & EV_PERSIST) != 0);
+        prepPollAdd(sqe, ev->ev_fd, getPollFlags(ev->ev_events));
       }
     }
 
@@ -630,7 +582,7 @@ class IoUringBackend : public EventBaseBackendBase {
             ret = true;
             std::unique_ptr<IOBuf> buf;
             if (flags & IORING_CQE_F_BUFFER) {
-              if (ProvidedBufferProviderBase* bp = backend->bufferProvider()) {
+              if (IoUringBufferProviderBase* bp = backend->bufferProvider()) {
                 buf = bp->getIoBuf(flags >> 16, res);
               }
             }
@@ -677,18 +629,9 @@ class IoUringBackend : public EventBaseBackendBase {
     EventCallbackData cbData_;
 
     void prepPollAdd(
-        struct io_uring_sqe* sqe, int fd, uint32_t events, bool registerFd) {
+        struct io_uring_sqe* sqe, int fd, uint32_t events) noexcept {
       CHECK(sqe);
-      if (registerFd && !fdRecord_) {
-        fdRecord_ = backend_->registerFd(fd);
-      }
-
-      if (fdRecord_) {
-        ::io_uring_prep_poll_add(sqe, fdRecord_->idx_, events);
-        sqe->flags |= IOSQE_FIXED_FILE;
-      } else {
-        ::io_uring_prep_poll_add(sqe, fd, events);
-      }
+      ::io_uring_prep_poll_add(sqe, fd, events);
       ::io_uring_sqe_set_data(sqe, this);
     }
 
@@ -697,7 +640,7 @@ class IoUringBackend : public EventBaseBackendBase {
         int fd,
         const struct iovec* iov,
         off_t offset,
-        bool registerFd) {
+        bool registerFd) noexcept {
       CHECK(sqe);
       if (registerFd && !fdRecord_) {
         fdRecord_ = backend_->registerFd(fd);
@@ -705,10 +648,15 @@ class IoUringBackend : public EventBaseBackendBase {
 
       if (fdRecord_) {
         ::io_uring_prep_read(
-            sqe, fdRecord_->idx_, iov->iov_base, iov->iov_len, offset);
+            sqe,
+            fdRecord_->idx_,
+            iov->iov_base,
+            (unsigned int)iov->iov_len,
+            offset);
         sqe->flags |= IOSQE_FIXED_FILE;
       } else {
-        ::io_uring_prep_read(sqe, fd, iov->iov_base, iov->iov_len, offset);
+        ::io_uring_prep_read(
+            sqe, fd, iov->iov_base, (unsigned int)iov->iov_len, offset);
       }
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -718,7 +666,7 @@ class IoUringBackend : public EventBaseBackendBase {
         int fd,
         const struct iovec* iov,
         off_t offset,
-        bool registerFd) {
+        bool registerFd) noexcept {
       CHECK(sqe);
       if (registerFd && !fdRecord_) {
         fdRecord_ = backend_->registerFd(fd);
@@ -726,16 +674,24 @@ class IoUringBackend : public EventBaseBackendBase {
 
       if (fdRecord_) {
         ::io_uring_prep_write(
-            sqe, fdRecord_->idx_, iov->iov_base, iov->iov_len, offset);
+            sqe,
+            fdRecord_->idx_,
+            iov->iov_base,
+            (unsigned int)iov->iov_len,
+            offset);
         sqe->flags |= IOSQE_FIXED_FILE;
       } else {
-        ::io_uring_prep_write(sqe, fd, iov->iov_base, iov->iov_len, offset);
+        ::io_uring_prep_write(
+            sqe, fd, iov->iov_base, (unsigned int)iov->iov_len, offset);
       }
       ::io_uring_sqe_set_data(sqe, this);
     }
 
     void prepRecvmsg(
-        struct io_uring_sqe* sqe, int fd, struct msghdr* msg, bool registerFd) {
+        struct io_uring_sqe* sqe,
+        int fd,
+        struct msghdr* msg,
+        bool registerFd) noexcept {
       CHECK(sqe);
       if (registerFd && !fdRecord_) {
         fdRecord_ = backend_->registerFd(fd);
@@ -751,24 +707,15 @@ class IoUringBackend : public EventBaseBackendBase {
     }
 
     void prepRecvmsgMultishot(
-        struct io_uring_sqe* sqe, int fd, struct msghdr* msg, bool registerFd) {
+        struct io_uring_sqe* sqe, int fd, struct msghdr* msg) noexcept {
       CHECK(sqe);
-      if (registerFd && !fdRecord_) {
-        fdRecord_ = backend_->registerFd(fd);
-      }
-
-      if (fdRecord_) {
-        ::io_uring_prep_recvmsg(sqe, fdRecord_->idx_, msg, MSG_TRUNC);
-        sqe->flags |= IOSQE_FIXED_FILE;
-      } else {
-        ::io_uring_prep_recvmsg(sqe, fd, msg, MSG_TRUNC);
-      }
+      ::io_uring_prep_recvmsg(sqe, fd, msg, MSG_TRUNC);
       // this magic value is set in io_uring_prep_recvmsg_multishot,
       // however this version of the library isn't available widely yet
       // so just hardcode it here
       constexpr uint16_t kMultishotFlag = 1U << 1;
       sqe->ioprio |= kMultishotFlag;
-      if (ProvidedBufferProviderBase* bp = backend_->bufferProvider()) {
+      if (IoUringBufferProviderBase* bp = backend_->bufferProvider()) {
         sqe->buf_group = bp->gid();
         sqe->flags |= IOSQE_BUFFER_SELECT;
       }
@@ -834,7 +781,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~ReadIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       prepRead(sqe, fd_, iov_.data(), offset_, false);
     }
   };
@@ -843,7 +790,7 @@ class IoUringBackend : public EventBaseBackendBase {
     using ReadWriteIoSqe::ReadWriteIoSqe;
     ~WriteIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       prepWrite(sqe, fd_, iov_.data(), offset_, false);
     }
   };
@@ -853,8 +800,9 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~ReadvIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
-      ::io_uring_prep_readv(sqe, fd_, iov_.data(), iov_.size(), offset_);
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
+      ::io_uring_prep_readv(
+          sqe, fd_, iov_.data(), (unsigned int)iov_.size(), offset_);
       ::io_uring_sqe_set_data(sqe, this);
     }
   };
@@ -863,8 +811,9 @@ class IoUringBackend : public EventBaseBackendBase {
     using ReadWriteIoSqe::ReadWriteIoSqe;
     ~WritevIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
-      ::io_uring_prep_writev(sqe, fd_, iov_.data(), iov_.size(), offset_);
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
+      ::io_uring_prep_writev(
+          sqe, fd_, iov_.data(), (unsigned int)iov_.size(), offset_);
       ::io_uring_sqe_set_data(sqe, this);
     }
   };
@@ -881,7 +830,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~FSyncIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       unsigned int fsyncFlags = 0;
       switch (flags_) {
         case FSyncFlags::FLAGS_FSYNC:
@@ -914,7 +863,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~FOpenAtIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_openat(sqe, fd_, path_.c_str(), flags_, mode_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -935,7 +884,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~FOpenAt2IoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_openat2(sqe, fd_, path_.c_str(), &how_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -949,7 +898,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~FCloseIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_close(sqe, fd_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -970,7 +919,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~FAllocateIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_fallocate(sqe, fd_, mode_, offset_, len_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -991,7 +940,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~SendmsgIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_sendmsg(sqe, fd_, msg_, flags_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -1011,7 +960,7 @@ class IoUringBackend : public EventBaseBackendBase {
 
     ~RecvmsgIoSqe() override = default;
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       ::io_uring_prep_recvmsg(sqe, fd_, msg_, flags_);
       ::io_uring_sqe_set_data(sqe, this);
     }
@@ -1025,21 +974,22 @@ class IoUringBackend : public EventBaseBackendBase {
   int submitOne();
   int cancelOne(IoSqe* ioSqe);
 
-  int submitBusyCheck(int num, WaitForEventsMode waitForEvents);
+  int submitBusyCheck(int num, WaitForEventsMode waitForEvents) noexcept;
   int submitEager();
 
   void queueFsync(int fd, FSyncFlags flags, FileOpCallback&& cb);
 
-  void processFileOp(IoSqe* ioSqe, int64_t res) noexcept;
+  void processFileOp(IoSqe* ioSqe, int res) noexcept;
 
   static void processFileOpCB(
-      IoUringBackend* backend, IoSqe* ioSqe, int64_t res, uint32_t) {
+      IoUringBackend* backend, IoSqe* ioSqe, int res, uint32_t) {
     static_cast<IoUringBackend*>(backend)->processFileOp(ioSqe, res);
   }
 
   IoUringBackend::IoSqe* allocNewIoSqe(const EventCallback& /*cb*/) {
-    // allow pool alloc if numIoSqeInUse_ < numEntries_
-    auto* ret = new IoSqe(this, numIoSqeInUse_ < numEntries_);
+    // allow pool alloc if numPooledIoSqeInUse_ < numEntries_
+    auto* ret = new IoSqe(this, numPooledIoSqeInUse_ < numEntries_);
+    ++numPooledIoSqeInUse_;
     ret->backendCb_ = IoUringBackend::processPollIoSqe;
 
     return ret;
@@ -1047,13 +997,23 @@ class IoUringBackend : public EventBaseBackendBase {
 
   void cleanup();
 
-  struct io_uring_sqe* get_sqe();
+  struct io_uring_sqe* getUntrackedSqe();
+  struct io_uring_sqe* getSqe();
+
+  /// some ring calls require being called on a single system thread, so we need
+  /// to delay init of those things until the correct thread is ready
+  void delayedInit();
+
+  /// init things that are linked to the io_uring submitter concept
+  /// so for DeferTaskrun, only do this in delayed init
+  void initSubmissionLinked();
 
   Options options_;
   size_t numEntries_;
   std::unique_ptr<IoSqe> timerEntry_;
   std::unique_ptr<IoSqe> signalReadEntry_;
   IoSqeList freeList_;
+  bool usingDeferTaskrun_{false};
 
   // timer related
   int timerFd_{-1};
@@ -1068,7 +1028,7 @@ class IoUringBackend : public EventBaseBackendBase {
   // submit
   IoSqeBaseList submitList_;
   uint16_t bufferProviderGidNext_{0};
-  std::unique_ptr<ProvidedBufferProviderBase> bufferProvider_;
+  IoUringBufferProviderBase::UniquePtr bufferProvider_;
 
   // loop related
   bool loopBreak_{false};
@@ -1076,11 +1036,12 @@ class IoUringBackend : public EventBaseBackendBase {
   bool processTimers_{false};
   bool processSignals_{false};
   IoSqeList activeEvents_;
-  // number of IoSqe instances in use
   size_t waitingToSubmit_{0};
   size_t numInsertedEvents_{0};
   size_t numInternalEvents_{0};
-  size_t numIoSqeInUse_{0};
+
+  // number of pooled IoSqe instances in use
+  size_t numPooledIoSqeInUse_{0};
 
   // io_uring related
   struct io_uring_params params_;
@@ -1092,21 +1053,23 @@ class IoUringBackend : public EventBaseBackendBase {
   // every time we poll for a CQE
   CQPollLoopCallback cqPollLoopCallback_;
 
-  bool registerDefaultFds_{true};
+  bool needsDelayedInit_{true};
 
   // stuff for ensuring we don't re-enter submit/getActiveEvents
+  folly::Optional<std::thread::id> submitTid_;
   int isSubmitting_{0};
   bool gettingEvents_{false};
-  void setSubmitting() { isSubmitting_++; }
-  void doneSubmitting() { isSubmitting_--; }
+  void dCheckSubmitTid();
+  void setSubmitting() noexcept { isSubmitting_++; }
+  void doneSubmitting() noexcept { isSubmitting_--; }
   void setGetActiveEvents() {
     if (kIsDebug && gettingEvents_) {
       throw std::runtime_error("getting events is not reentrant");
       gettingEvents_ = true;
     }
   }
-  void doneGetActiveEvents() { gettingEvents_ = false; }
-  bool isSubmitting() const { return isSubmitting_; }
+  void doneGetActiveEvents() noexcept { gettingEvents_ = false; }
+  bool isSubmitting() const noexcept { return isSubmitting_; }
 };
 
 using PollIoBackend = IoUringBackend;
