@@ -147,7 +147,7 @@ class EventFD : public folly::EventHandler, public folly::EventReadCallback {
   uint64_t getNum() const { return num_; }
 
   // from folly::EventReadCallback
-  folly::EventReadCallback::IoVec* allocateData() override {
+  folly::EventReadCallback::IoVec* allocateData() noexcept override {
     auto* ret = ioVecPtr_.release();
     return (ret ? ret : new IoVec(this));
   }
@@ -407,7 +407,7 @@ class EventRecvmsgCallback : public folly::EventRecvmsgCallback {
   ~EventRecvmsgCallback() override = default;
 
   // from EventRecvmsgCallback
-  EventRecvmsgCallback::MsgHdr* allocateData() override {
+  EventRecvmsgCallback::MsgHdr* allocateData() noexcept override {
     auto* ret = msgHdr_.release();
     if (!ret) {
       ret = new MsgHdr(this);
@@ -501,8 +501,8 @@ class EventRecvmsgMultishotCallback
   ~EventRecvmsgMultishotCallback() override = default;
 
   // from EventRecvmsgCallback
-  folly::EventRecvmsgMultishotCallback::Hdr* allocateRecvmsgMultishotData()
-      override {
+  folly::EventRecvmsgMultishotCallback::Hdr*
+  allocateRecvmsgMultishotData() noexcept override {
     return new Hdr(this);
   }
 
@@ -537,8 +537,10 @@ void testAsyncUDPRecvmsg(bool useRegisteredFds, bool multishot = false) {
   }
   auto evbPtr = getEventBase(options);
   SKIP_IF(!evbPtr) << "Backend not available";
-  SKIP_IF(multishot && !folly::IoUringBackend::kernelSupportsRecvmsgMultishot())
-      << "multishot not available";
+  if (multishot && !folly::IoUringBackend::kernelSupportsRecvmsgMultishot()) {
+    LOG(INFO) << "multishot not available";
+    return;
+  }
 
   // create the server sockets
   std::vector<std::unique_ptr<folly::AsyncUDPServerSocket>> serverSocketVec;
@@ -833,11 +835,12 @@ TEST(IoUringBackend, RegisteredFds) {
 
   SCOPE_EXIT { ::close(eventFd); };
 
-  // verify for useRegisteredFds = false we get a nullptr FdRegistrationRecord
+  // verify for useRegisteredFds = false we get a nullptr
+  // IoUringFdRegistrationRecord
   auto* record = backendNoReg->registerFd(eventFd);
   CHECK(!record);
 
-  std::vector<folly::IoUringBackend::FdRegistrationRecord*> records;
+  std::vector<folly::IoUringFdRegistrationRecord*> records;
   // we use kBackendCapacity since the timerFd
   // allocates it only on the first loop
   records.reserve(kBackendCapacity);
@@ -1274,19 +1277,21 @@ TEST(IoUringBackend, ProvidedBuffers) {
 
   EXPECT_EQ(2, bufferProvider->count());
 
-  struct Reader : folly::IoUringBackend::IoSqeBase {
+  struct Reader : folly::IoSqeBase {
     Reader(int fd, uint16_t bgid, std::function<void(int, uint32_t)> oncqe)
         : fd_(fd), bgid_(bgid), oncqe_(oncqe) {}
 
-    void processSubmit(struct io_uring_sqe* sqe) override {
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
       io_uring_prep_read(sqe, fd_, nullptr, 2 /* max read 2 per go */, 0);
       sqe->flags |= IOSQE_BUFFER_SELECT;
       sqe->buf_group = bgid_;
     }
 
-    void callback(int res, uint32_t flags) override { oncqe_(res, flags); }
+    void callback(int res, uint32_t flags) noexcept override {
+      oncqe_(res, flags);
+    }
 
-    void callbackCancelled() override { FAIL(); }
+    void callbackCancelled(int, uint32_t) noexcept override { FAIL(); }
 
     int fd_;
     uint16_t bgid_;
@@ -1332,13 +1337,103 @@ TEST(IoUringBackend, ProvidedBuffers) {
   // now the buffers should be back
   readers.clear();
   cqes.clear();
-  addReaders(2);
+  addReaders(1);
   backend->eb_event_base_loop(EVLOOP_ONCE);
   ASSERT_EQ(1, cqes.size());
   EXPECT_EQ(2, cqes[0].first);
   EXPECT_EQ("56", toString(bufferProvider->getIoBuf(cqes[0].second >> 16, 2)));
 }
 
+TEST(IoUringBackend, ProvidedBufferRing) {
+  auto evbPtr = getEventBase();
+  int constexpr kBuffs = 3;
+  for (int keep = 0; keep <= kBuffs; keep++) {
+    std::vector<std::unique_ptr<folly::IOBuf>> bufs;
+    std::unique_ptr<folly::IoUringBackend> backend;
+    try {
+      backend = std::make_unique<folly::IoUringBackend>(
+          folly::IoUringBackend::Options{}.setInitialProvidedBuffers(
+              1024, kBuffs));
+    } catch (folly::IoUringBackend::NotAvailable const&) {
+      return;
+    }
+
+    auto* bufferProvider = backend->bufferProvider();
+    ASSERT_NE(bufferProvider, nullptr);
+    for (int i = 0; i < 16; i++) {
+      bufferProvider->getIoBuf(i % kBuffs, 1);
+    }
+    bufferProvider->unusedBuf(0);
+    for (int i = 0; i < keep; i++) {
+      bufs.push_back(bufferProvider->getIoBuf(i % kBuffs, 1));
+    }
+  }
+}
+
+TEST(IoUringBackend, BigProvidedBufferRing) {
+  auto evbPtr = getEventBase();
+  // test that big buffer rings don't have memory corruption
+  int constexpr kBuffs = 32000;
+  int constexpr kSize = 100000;
+
+  std::unique_ptr<folly::IoUringBackend> backend;
+  try {
+    backend = std::make_unique<folly::IoUringBackend>(
+        folly::IoUringBackend::Options{}.setInitialProvidedBuffers(
+            kSize, kBuffs));
+  } catch (folly::IoUringBackend::NotAvailable const&) {
+    return;
+  }
+
+  auto* bufferProvider = backend->bufferProvider();
+  ASSERT_NE(bufferProvider, nullptr);
+  for (int i = 0; i < kBuffs; i++) {
+    // test that we can obtain all the possible buffers and return them
+    auto buff = bufferProvider->getIoBuf(i, kSize);
+    memset(buff->writableData(), 0, buff->length());
+  }
+}
+
+TEST(IoUringBackend, DeferTaskRun) {
+  if (!folly::IoUringBackend::kernelSupportsDeferTaskrun()) {
+    return;
+  }
+
+  std::atomic<int> doneA{0};
+  std::atomic<int> doneB{0};
+  struct N : folly::IoSqeBase {
+    explicit N(std::atomic<int>& v) : val(v) {}
+    std::atomic<int>& val;
+    void processSubmit(struct io_uring_sqe* sqe) noexcept override {
+      ::io_uring_prep_nop(sqe);
+    }
+    void callback(int, uint32_t) noexcept override {
+      ++val;
+      delete this;
+    }
+    void callbackCancelled(int, uint32_t) noexcept override {
+      ++val;
+      delete this;
+    }
+  };
+
+  N* maybeLeaks = nullptr;
+
+  std::unique_ptr<folly::IoUringBackend> backend;
+  std::async(std::launch::async, [&]() {
+    backend = std::make_unique<folly::IoUringBackend>(
+        folly::IoUringBackend::Options().setDeferTaskRun(true));
+    backend->submitNow(*new N(doneA));
+    backend->loopPoll();
+    maybeLeaks = new N(doneB);
+    backend->submitSoon(*maybeLeaks);
+  }).wait();
+  backend.reset();
+  EXPECT_EQ(1, doneA.load());
+  ASSERT_EQ(0, doneB.load()) << "could not run on other thread";
+  delete maybeLeaks;
+}
+
 namespace folly {
 namespace test {
 static constexpr size_t kCapacity = 32;
