@@ -51,6 +51,8 @@ using ElfOff = FOLLY_ELF_ELFW(Off);
 using ElfPhdr = FOLLY_ELF_ELFW(Phdr);
 using ElfShdr = FOLLY_ELF_ELFW(Shdr);
 using ElfSym = FOLLY_ELF_ELFW(Sym);
+using ElfRel = FOLLY_ELF_ELFW(Rel);
+using ElfRela = FOLLY_ELF_ELFW(Rela);
 
 // ElfFileId is supposed to uniquely identify any instance of an ELF binary.
 // It does that by using file's inode, dev ID, size and modification time:
@@ -194,7 +196,7 @@ class ElfFile {
       noexcept(is_nothrow_invocable_v<Fn, ElfShdr const&>);
 
   /**
-   * Iterate over all symbols witin a given section.
+   * Iterate over all symbols within a given section.
    *
    * Returns a pointer to the current ("found") symbol when fn returned true,
    * or nullptr if fn returned false for all symbols.
@@ -212,6 +214,17 @@ class ElfFile {
       std::initializer_list<uint32_t> types,
       Fn fn) const noexcept(is_nothrow_invocable_v<Fn, ElfSym const&>);
 
+  /**
+   * Iterate over entries within a given section.
+   *
+   * Returns a pointer to the current ("found") entry when fn returned
+   * true, or nullptr if fn returned false for all entries.
+   */
+  template <typename E>
+  const E* iterateSectionEntries(
+      const ElfShdr& section, std::function<bool(const E&)> fn) const
+      noexcept(is_nothrow_invocable_v<E const&>);
+
   /**
    * Find symbol definition by address.
    * Note that this is the file virtual address, so you need to undo
@@ -223,14 +236,18 @@ class ElfFile {
   Symbol getDefinitionByAddress(uintptr_t address) const noexcept;
 
   /**
-   * Find symbol definition by name.
+   * Find symbol definition by name. Optionally specify the symbol types to
+   * consider.
    *
    * If a symbol with this name cannot be found, a <nullptr, nullptr> Symbol
    * will be returned. This is O(N) in the number of symbols in the file.
    *
    * Returns {nullptr, nullptr} if not found.
    */
-  Symbol getSymbolByName(const char* name) const noexcept;
+  Symbol getSymbolByName(
+      const char* name,
+      std::initializer_list<uint32_t> types = {
+          STT_OBJECT, STT_FUNC, STT_GNU_IFUNC}) const noexcept;
 
   /**
    * Get the value of a symbol.
@@ -329,6 +346,15 @@ class ElfFile {
             (addr + sizeof(T)) <= (section.sh_addr + section.sh_size),
         "Address is not contained within the provided segment");
 
+    // SHT_NOBITS: a section that occupies no space in the file but otherwise
+    // resembles SHT_PROGBITS. Although this section contains no bytes, the
+    // sh_offset member contains the conceptual file offset. Typically used
+    // for zero-initialized data sections like .bss.
+    if (section.sh_type == SHT_NOBITS) {
+      static T t = {};
+      return t;
+    }
+
     return at<T>(section.sh_offset + (addr - section.sh_addr));
   }
 
