@@ -393,7 +393,7 @@ Future<T> chainExecutor(Executor::KeepAlive<> e, SemiFuture<T>&& f) {
   if (!e) {
     e = folly::getKeepAliveToken(InlineExecutor::instance());
   }
-  return std::move(f).via(e);
+  return std::move(f).via(std::move(e));
 }
 
 // Variant: returns a Future
@@ -1475,7 +1475,7 @@ collect(InputIterator first, InputIterator last) {
   auto future = ctx->p.getSemiFuture();
   if (!executors.empty()) {
     auto work = [](Try<typename decltype(future)::value_type>&& t) {
-      return std::move(t).value();
+      return std::move(t);
     };
     future = std::move(future).defer(work);
     const auto& deferredExecutor = futures::detail::getDeferredExecutor(future);
