@@ -141,14 +141,13 @@ struct IOBuf::HeapStorage {
   folly::IOBuf buf;
 };
 
-struct IOBuf::HeapFullStorage {
+struct alignas(folly::max_align_v) IOBuf::HeapFullStorage {
   // Make sure jemalloc allocates from the 64-byte class.  Putting this here
   // because HeapStorage is private so it can't be at namespace level.
   static_assert(sizeof(HeapStorage) <= 64, "IOBuf may not grow over 56 bytes!");
 
   HeapStorage hs;
   SharedInfo shared;
-  folly::max_align_t align;
 };
 
 IOBuf::SharedInfo::SharedInfo()
@@ -225,9 +224,12 @@ void IOBuf::operator delete(void* /* ptr */, void* /* placement */) {
 void IOBuf::releaseStorage(HeapStorage* storage, uint16_t freeFlags) noexcept {
   CHECK_EQ(storage->prefix.magic, static_cast<uint16_t>(kHeapMagic));
 
-  // Use relaxed memory order here.  If we are unlucky and happen to get
-  // out-of-date data the compare_exchange_weak() call below will catch
-  // it and load new data with memory_order_acq_rel.
+  // This function is effectively used as a memory barrier.  Logically, we can
+  // use relaxed memory order here.  If we are unlucky and happen to get
+  // out-of-date data the compare_exchange_weak() call below will catch it and
+  // load new data with memory_order_acq_rel.  However, changing this to be
+  // std::memory_order_relaxed will cause tsan to find problems with some
+  // caller code.
   auto flags = storage->prefix.flags.load(std::memory_order_acquire);
   DCHECK_EQ((flags & freeFlags), freeFlags);
 
@@ -360,7 +362,7 @@ unique_ptr<IOBuf> IOBuf::createCombined(std::size_t capacity) {
 
   // To save a memory allocation, allocate space for the IOBuf object, the
   // SharedInfo struct, and the data itself all with a single call to malloc().
-  size_t requiredStorage = offsetof(HeapFullStorage, align) + capacity;
+  size_t requiredStorage = sizeof(HeapFullStorage) + capacity;
   size_t mallocSize = goodMallocSize(requiredStorage);
   auto storage = static_cast<HeapFullStorage*>(checkedMalloc(mallocSize));
 
@@ -371,7 +373,7 @@ unique_ptr<IOBuf> IOBuf::createCombined(std::size_t capacity) {
     io_buf_alloc_cb(storage, mallocSize);
   }
 
-  auto bufAddr = reinterpret_cast<uint8_t*>(&storage->align);
+  auto bufAddr = reinterpret_cast<uint8_t*>(storage) + sizeof(HeapFullStorage);
   uint8_t* storageEnd = reinterpret_cast<uint8_t*>(storage) + mallocSize;
   auto actualCapacity = size_t(storageEnd - bufAddr);
   unique_ptr<IOBuf> ret(new (&storage->hs.buf) IOBuf(
@@ -412,7 +414,7 @@ size_t IOBuf::goodSize(size_t minCapacity, CombinedOption combined) {
   }
   size_t overhead;
   if (combined == CombinedOption::COMBINED) {
-    overhead = offsetof(HeapFullStorage, align);
+    overhead = sizeof(HeapFullStorage);
   } else {
     // Pad minCapacity to a multiple of 8
     minCapacity = (minCapacity + 7) & ~7;
@@ -612,7 +614,10 @@ IOBuf::IOBuf(
   assert(data >= buf);
   assert(intptr_t(data) + length <= intptr_t(buf) + capacity);
 
-  CHECK(!folly::asan_region_is_poisoned(buf, capacity));
+  if (folly::asan_region_is_poisoned(buf, capacity)) {
+    // If we know it's a poisoned region, access it to trigger ASAN reporting.
+    memset(buf, 0, capacity);
+  }
 }
 
 IOBuf::~IOBuf() {
