@@ -1453,7 +1453,7 @@ void AsyncSSLSocket::invokeConnectErr(const AsyncSocketException& ex) {
 void AsyncSSLSocket::invokeConnectSuccess() {
   connectionTimeout_.cancelTimeout();
   if (sslState_ == SSLStateEnum::STATE_CONNECTING) {
-    assert(tfoAttempted_);
+    assert(tfoInfo_.attempted);
     // If we failed TFO, we'd fall back to trying to connect the socket,
     // to setup things like timeouts.
     startSSLConnect();
@@ -1624,7 +1624,11 @@ AsyncSocket::ReadResult AsyncSSLSocket::performReadv(
       } else if (ReadResultEnum(res) == READ_ERROR) {
         return readRes;
       } else if (ReadResultEnum(res) == READ_BLOCKING) {
-        return ReadResult(totalRead);
+        if (totalRead > 0) {
+          return ReadResult(totalRead);
+        } else {
+          return readRes;
+        }
       }
     }
   }
@@ -2221,8 +2225,8 @@ void AsyncSSLSocket::getSSLClientCiphers(
     if (!nameFound) {
       folly::hexlify(
           std::array<uint8_t, 2>{
-              {static_cast<uint8_t>((originalCipherCode >> 8) & 0xffL),
-               static_cast<uint8_t>(originalCipherCode & 0x00ffL)}},
+              {static_cast<uint8_t>((originalCipherCode >> 8) & 0xff),
+               static_cast<uint8_t>(originalCipherCode & 0x00ff)}},
           ciphers,
           /* append to ciphers = */ true);
     }
