@@ -42,6 +42,20 @@
 #define FOLLY_HAVE_VLA_01 0
 #endif
 
+// xplat UDP GSO socket options.
+#ifdef _WIN32
+#ifndef UDP_SEND_MSG_SIZE
+#define UDP_SEND_MSG_SIZE 2
+#endif
+#define UDP_GSO_SOCK_OPT_LEVEL IPPROTO_UDP
+#define UDP_GSO_SOCK_OPT_TYPE UDP_SEND_MSG_SIZE
+#define GSO_OPT_TYPE DWORD
+#else /* !_WIN32 */
+#define UDP_GSO_SOCK_OPT_LEVEL SOL_UDP
+#define UDP_GSO_SOCK_OPT_TYPE UDP_SEGMENT
+#define GSO_OPT_TYPE uint16_t
+#endif /* _WIN32 */
+
 namespace fsp = folly::portability::sockets;
 
 namespace folly {
@@ -59,18 +73,20 @@ void AsyncUDPSocket::fromMsg(
         grosizeptr = (uint16_t*)CMSG_DATA(cmsg);
         params.gro = *grosizeptr;
       }
-    } else {
-      if (cmsg->cmsg_level == SOL_SOCKET) {
-        if (cmsg->cmsg_type == SO_TIMESTAMPING ||
-            cmsg->cmsg_type == SO_TIMESTAMPNS) {
-          ReadCallback::OnDataAvailableParams::Timestamp ts;
-          memcpy(
-              &ts,
-              reinterpret_cast<struct timespec*>(CMSG_DATA(cmsg)),
-              sizeof(ts));
-          params.ts = ts;
-        }
+    } else if (cmsg->cmsg_level == SOL_SOCKET) {
+      if (cmsg->cmsg_type == SO_TIMESTAMPING ||
+          cmsg->cmsg_type == SO_TIMESTAMPNS) {
+        ReadCallback::OnDataAvailableParams::Timestamp ts;
+        memcpy(
+            &ts,
+            reinterpret_cast<struct timespec*>(CMSG_DATA(cmsg)),
+            sizeof(ts));
+        params.ts = ts;
       }
+    } else if (
+        (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_TOS) ||
+        (cmsg->cmsg_level == SOL_IPV6 && cmsg->cmsg_type == IPV6_TCLASS)) {
+      params.tos = *(uint8_t*)CMSG_DATA(cmsg);
     }
   }
 #endif
@@ -83,11 +99,10 @@ static constexpr bool msgErrQueueSupported =
 #endif // FOLLY_HAVE_MSG_ERRQUEUE
 
 AsyncUDPSocket::AsyncUDPSocket(EventBase* evb)
-    : EventHandler(CHECK_NOTNULL(evb)),
-      readCallback_(nullptr),
-      eventBase_(evb),
-      fd_() {
-  evb->dcheckIsInEventBaseThread();
+    : EventHandler(evb), readCallback_(nullptr), eventBase_(evb), fd_() {
+  if (eventBase_) {
+    eventBase_->dcheckIsInEventBaseThread();
+  }
 }
 
 AsyncUDPSocket::~AsyncUDPSocket() {
@@ -97,6 +112,11 @@ AsyncUDPSocket::~AsyncUDPSocket() {
 }
 
 void AsyncUDPSocket::init(sa_family_t family, BindOptions bindOptions) {
+  if (fd_ != NetworkSocket()) {
+    // Already initialized.
+    return;
+  }
+
   NetworkSocket socket =
       netops::socket(family, SOCK_DGRAM, family != AF_UNIX ? IPPROTO_UDP : 0);
   if (socket == NetworkSocket()) {
@@ -230,6 +250,29 @@ void AsyncUDPSocket::init(sa_family_t family, BindOptions bindOptions) {
     }
   }
 
+  if (recvTos_) {
+    // Set socket option to receive IPv6 Traffic Class/IPv4 Type of Service.
+    int flag = 1;
+    if (family == AF_INET6) {
+      if (netops::setsockopt(
+              socket, IPPROTO_IPV6, IPV6_RECVTCLASS, &flag, sizeof(flag)) !=
+          0) {
+        throw AsyncSocketException(
+            AsyncSocketException::NOT_OPEN,
+            "failed to set IPV6_RECVTCLASS on the socket",
+            errno);
+      }
+    } else if (family == AF_INET) {
+      if (netops::setsockopt(
+              socket, IPPROTO_IP, IP_RECVTOS, &flag, sizeof(flag)) != 0) {
+        throw AsyncSocketException(
+            AsyncSocketException::NOT_OPEN,
+            "failed to set IP_RECVTOS on the socket",
+            errno);
+      }
+    }
+  }
+
   if (family == AF_INET6) {
     int flag = static_cast<int>(bindOptions.bindV6Only);
     if (netops::setsockopt(
@@ -250,7 +293,9 @@ void AsyncUDPSocket::init(sa_family_t family, BindOptions bindOptions) {
 
 void AsyncUDPSocket::bind(
     const folly::SocketAddress& address, BindOptions bindOptions) {
-  init(address.getFamily(), bindOptions);
+  if (fd_ == NetworkSocket()) {
+    init(address.getFamily(), bindOptions);
+  }
 
   {
     // bind the socket to the interface
@@ -291,7 +336,7 @@ void AsyncUDPSocket::bind(
 void AsyncUDPSocket::connect(const folly::SocketAddress& address) {
   // not bound yet
   if (fd_ == NetworkSocket()) {
-    init(address.getFamily(), BindOptions());
+    init(address.getFamily());
   }
 
   sockaddr_storage addrStorage;
@@ -621,28 +666,25 @@ ssize_t AsyncUDPSocket::writev(
     size_t iovec_len,
     int gso) {
   CHECK_NE(NetworkSocket(), fd_) << "Socket not yet bound";
+  netops::Msgheader msg;
   sockaddr_storage addrStorage;
   address.getAddress(&addrStorage);
 
-  struct msghdr msg;
   if (!connected_) {
-    msg.msg_name = reinterpret_cast<void*>(&addrStorage);
-    msg.msg_namelen = address.getActualSize();
+    msg.setName(&addrStorage, address.getActualSize());
   } else {
     if (connectedAddress_ != address) {
       errno = ENOTSUP;
       return -1;
     }
-    msg.msg_name = nullptr;
-    msg.msg_namelen = 0;
+    msg.setName(nullptr, 0);
   }
-  msg.msg_iov = const_cast<struct iovec*>(vec);
-  msg.msg_iovlen = iovec_len;
-  msg.msg_control = nullptr;
-  msg.msg_controllen = 0;
-  msg.msg_flags = 0;
+  msg.setIovecs(vec, iovec_len);
+  msg.setCmsgPtr(nullptr);
+  msg.setCmsgLen(0);
+  msg.setFlags(0);
 
-#ifdef FOLLY_HAVE_MSG_ERRQUEUE
+#if defined(FOLLY_HAVE_MSG_ERRQUEUE) || defined(_WIN32)
   constexpr size_t kSmallSizeMax = 5;
   size_t controlBufSize = gso > 0 ? 1 : 0;
   controlBufSize +=
@@ -657,7 +699,7 @@ ssize_t AsyncUDPSocket::writev(
         [(BOOST_PP_IF(FOLLY_HAVE_VLA_01, controlBufSize, kSmallSizeMax)) *
          (CMSG_SPACE(sizeof(uint16_t)))];
     memset(control, 0, sizeof(control));
-    msg.msg_control = control;
+    msg.setCmsgPtr(control);
     FOLLY_POP_WARNING
     return writevImpl(&msg, gso);
   } else {
@@ -667,70 +709,68 @@ ssize_t AsyncUDPSocket::writev(
     }
     std::unique_ptr<char[]> control(new char[controlBufSize]);
     memset(control.get(), 0, controlBufSize);
-    msg.msg_control = control.get();
+    msg.setCmsgPtr(control.get());
     return writevImpl(&msg, gso);
   }
 #else
   CHECK_LT(gso, 1) << "GSO not supported";
 #endif
 
-  return sendmsg(fd_, &msg, 0);
+#ifdef _WIN32
+  return netops::wsaSendMsgDirect(fd_, msg.getMsg());
+#else
+  return sendmsg(fd_, msg.getMsg(), 0);
+#endif
 }
 
 ssize_t AsyncUDPSocket::writevImpl(
-    struct msghdr* msg, FOLLY_MAYBE_UNUSED int gso) {
-#ifdef FOLLY_HAVE_MSG_ERRQUEUE
-  struct cmsghdr* cm = nullptr;
+    netops::Msgheader* msg, FOLLY_MAYBE_UNUSED int gso) {
+#if defined(FOLLY_HAVE_MSG_ERRQUEUE) || defined(_WIN32)
+  XPLAT_CMSGHDR* cm = nullptr;
+
   for (auto itr = cmsgs_.begin(); itr != cmsgs_.end(); ++itr) {
     const auto key = itr->first;
     const auto val = itr->second;
-    msg->msg_controllen += CMSG_SPACE(sizeof(val));
-    if (cm) {
-      cm = CMSG_NXTHDR(msg, cm);
-    } else {
-      cm = CMSG_FIRSTHDR(msg);
-    }
+    msg->incrCmsgLen(sizeof(val));
+    cm = msg->getFirstOrNextCmsgHeader(cm);
     if (cm) {
       cm->cmsg_level = key.level;
       cm->cmsg_type = key.optname;
-      cm->cmsg_len = CMSG_LEN(sizeof(val));
-      memcpy(CMSG_DATA(cm), &val, sizeof(val));
+      cm->cmsg_len = F_CMSG_LEN(sizeof(val));
+      F_COPY_CMSG_INT_DATA(cm, &val, sizeof(val));
     }
   }
   for (const auto& itr : nontrivialCmsgs_) {
     const auto& key = itr.first;
     const auto& val = itr.second;
-    msg->msg_controllen += CMSG_SPACE(val.size());
-    if (cm) {
-      cm = CMSG_NXTHDR(msg, cm);
-    } else {
-      cm = CMSG_FIRSTHDR(msg);
-    }
+    msg->incrCmsgLen(val.size());
+    cm = msg->getFirstOrNextCmsgHeader(cm);
     if (cm) {
       cm->cmsg_level = key.level;
       cm->cmsg_type = key.optname;
-      cm->cmsg_len = CMSG_LEN(val.size());
-      memcpy(CMSG_DATA(cm), val.data(), val.size());
+      cm->cmsg_len = F_CMSG_LEN(val.size());
+      F_COPY_CMSG_INT_DATA(cm, val.data(), val.size());
     }
   }
 
   if (gso > 0) {
-    msg->msg_controllen += CMSG_SPACE(sizeof(uint16_t));
+    msg->incrCmsgLen(sizeof(uint16_t));
+    cm = msg->getFirstOrNextCmsgHeader(cm);
     if (cm) {
-      cm = CMSG_NXTHDR(msg, cm);
-    } else {
-      cm = CMSG_FIRSTHDR(msg);
-    }
-    if (cm) {
-      cm->cmsg_level = SOL_UDP;
-      cm->cmsg_type = UDP_SEGMENT;
-      cm->cmsg_len = CMSG_LEN(sizeof(uint16_t));
-      auto gso_len = static_cast<uint16_t>(gso);
-      memcpy(CMSG_DATA(cm), &gso_len, sizeof(gso_len));
+      cm->cmsg_level = UDP_GSO_SOCK_OPT_LEVEL;
+      cm->cmsg_type = UDP_GSO_SOCK_OPT_TYPE;
+      cm->cmsg_len = F_CMSG_LEN(sizeof(GSO_OPT_TYPE));
+      auto gso_len = static_cast<GSO_OPT_TYPE>(gso);
+      F_COPY_CMSG_INT_DATA(cm, &gso_len, sizeof(gso_len));
     }
   }
-#endif // FOLLY_HAVE_MSG_ERRQUEUE
-  return sendmsg(fd_, msg, 0);
+#endif // FOLLY_HAVE_MSG_ERRQUEUE || _WIN32
+
+#ifdef _WIN32
+  return netops::wsaSendMsgDirect(fd_, msg->getMsg());
+#else
+  return sendmsg(fd_, msg->getMsg(), 0);
+#endif
 }
 
 ssize_t AsyncUDPSocket::writev(
@@ -1183,7 +1223,7 @@ void AsyncUDPSocket::handleRead() noexcept {
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
     bool use_gro = gro_.has_value() && (gro_.value() > 0);
     bool use_ts = ts_.has_value() && (ts_.value() > 0);
-    if (use_gro || use_ts) {
+    if (use_gro || use_ts || recvTos_) {
       char control[ReadCallback::OnDataAvailableParams::kCmsgSpace] = {};
 
       struct msghdr msg = {};
@@ -1260,8 +1300,9 @@ bool AsyncUDPSocket::updateRegistration() noexcept {
 }
 
 bool AsyncUDPSocket::setGSO(int val) {
-#ifdef FOLLY_HAVE_MSG_ERRQUEUE
-  int ret = netops::setsockopt(fd_, SOL_UDP, UDP_SEGMENT, &val, sizeof(val));
+#if defined(FOLLY_HAVE_MSG_ERRQUEUE) || defined(_WIN32)
+  int ret = netops::setsockopt(
+      fd_, UDP_GSO_SOCK_OPT_LEVEL, UDP_GSO_SOCK_OPT_TYPE, &val, sizeof(val));
 
   gso_ = ret ? -1 : val;
 
@@ -1275,10 +1316,15 @@ bool AsyncUDPSocket::setGSO(int val) {
 int AsyncUDPSocket::getGSO() {
   // check if we can return the cached value
   if (FOLLY_UNLIKELY(!gso_.has_value())) {
-#ifdef FOLLY_HAVE_MSG_ERRQUEUE
+#if defined(FOLLY_HAVE_MSG_ERRQUEUE) || defined(_WIN32)
     int gso = -1;
     socklen_t optlen = sizeof(gso);
-    if (!netops::getsockopt(fd_, SOL_UDP, UDP_SEGMENT, &gso, &optlen)) {
+    if (!netops::getsockopt(
+            fd_,
+            UDP_GSO_SOCK_OPT_LEVEL,
+            UDP_GSO_SOCK_OPT_TYPE,
+            &gso,
+            &optlen)) {
       gso_ = gso;
     } else {
       gso_ = -1;
@@ -1423,14 +1469,21 @@ bool AsyncUDPSocket::setTxZeroChksum6(FOLLY_MAYBE_UNUSED bool bVal) {
 #endif
 }
 
-void AsyncUDPSocket::setTrafficClass(int tclass) {
+void AsyncUDPSocket::setTrafficClass(uint8_t tclass) {
   if (netops::setsockopt(
-          fd_, IPPROTO_IPV6, IPV6_TCLASS, &tclass, sizeof(int)) != 0) {
+          fd_, IPPROTO_IPV6, IPV6_TCLASS, &tclass, sizeof(tclass)) != 0) {
     throw AsyncSocketException(
         AsyncSocketException::NOT_OPEN, "Failed to set IPV6_TCLASS", errno);
   }
 }
 
+void AsyncUDPSocket::setTos(uint8_t tos) {
+  if (netops::setsockopt(fd_, IPPROTO_IP, IP_TOS, &tos, sizeof(tos)) != 0) {
+    throw AsyncSocketException(
+        AsyncSocketException::NOT_OPEN, "Failed to set IP_TOS", errno);
+  }
+}
+
 void AsyncUDPSocket::applyOptions(
     const SocketOptionMap& options, SocketOptionKey::ApplyPos pos) {
   auto result = applySocketOptions(fd_, options, pos);
