@@ -51,10 +51,11 @@ class AsyncUDPSocket : public EventHandler {
       // RX timestamp if available
       using Timestamp = std::array<struct timespec, 3>;
       folly::Optional<Timestamp> ts;
+      uint8_t tos = 0;
 
 #ifdef FOLLY_HAVE_MSG_ERRQUEUE
-      static constexpr size_t kCmsgSpace =
-          CMSG_SPACE(sizeof(uint16_t)) + CMSG_SPACE(sizeof(Timestamp));
+      static constexpr size_t kCmsgSpace = CMSG_SPACE(sizeof(uint16_t)) +
+          CMSG_SPACE(sizeof(Timestamp)) + CMSG_SPACE(sizeof(uint8_t));
 #endif
     };
 
@@ -352,6 +353,12 @@ class AsyncUDPSocket : public EventHandler {
    */
   virtual void setTransparent(bool transparent) { transparent_ = transparent; }
 
+  /**
+   * Set IPV6_RECVTCLASS/IP_RECVTOS to allow receiving of the IPv6 Traffic
+   * Class/IPv4 Type of Service field.
+   */
+  virtual void setRecvTos(bool recvTos) { recvTos_ = recvTos; }
+
   /**
    * Set reuse port mode to call bind() on the same address multiple times
    */
@@ -466,7 +473,9 @@ class AsyncUDPSocket : public EventHandler {
   // disable/enable TX zero checksum for UDP over IPv6
   bool setTxZeroChksum6(bool bVal);
 
-  void setTrafficClass(int tclass);
+  void setTrafficClass(uint8_t tclass);
+
+  void setTos(uint8_t tos);
 
   virtual void applyOptions(
       const SocketOptionMap& options, SocketOptionKey::ApplyPos pos);
@@ -496,6 +505,12 @@ class AsyncUDPSocket : public EventHandler {
     return netops_.getOverride();
   }
 
+  // Initializes underlying socket fd. This is called in bind() and connect()
+  // internally if fd is not yet set at the time of the call. But if there is a
+  // need to apply socket options pre-bind, one can call this function
+  // explicitly before bind()/connect() and socket opts application.
+  void init(sa_family_t family, BindOptions bindOptions = BindOptions());
+
  protected:
   struct full_sockaddr_storage {
     sockaddr_storage storage;
@@ -533,7 +548,8 @@ class AsyncUDPSocket : public EventHandler {
       const int* gso,
       char* control);
 
-  virtual ssize_t writevImpl(struct msghdr* msg, FOLLY_MAYBE_UNUSED int gso);
+  virtual ssize_t writevImpl(
+      netops::Msgheader* msg, FOLLY_MAYBE_UNUSED int gso);
 
   size_t handleErrMessages() noexcept;
 
@@ -547,8 +563,6 @@ class AsyncUDPSocket : public EventHandler {
   ReadCallback* readCallback_;
 
  private:
-  void init(sa_family_t family, BindOptions bindOptions);
-
   // EventHandler
   void handlerReady(uint16_t events) noexcept override;
 
@@ -572,6 +586,7 @@ class AsyncUDPSocket : public EventHandler {
   bool reusePort_{false};
   bool freeBind_{false};
   bool transparent_{false};
+  bool recvTos_{false};
   int rcvBuf_{0};
   int sndBuf_{0};
   int busyPollUs_{0};
