@@ -39,6 +39,7 @@
 #include <folly/Portability.h>
 #include <folly/ScopeGuard.h>
 #include <folly/Synchronized.h>
+#include <folly/container/F14Set.h>
 #include <folly/executors/DrivableExecutor.h>
 #include <folly/executors/IOExecutor.h>
 #include <folly/executors/ScheduledExecutor.h>
@@ -216,7 +217,9 @@ class EventBase : public TimeoutManager,
   //   - Users may safely cancel() from any thread. Multiple calls to cancel()
   //     may execute concurrently. The only caveat is that it is not safe to
   //     call cancel() within the onEventBaseDestruction() callback.
-  class OnDestructionCallback {
+  class OnDestructionCallback
+      : public boost::intrusive::list_base_hook<
+            boost::intrusive::link_mode<boost::intrusive::normal_link>> {
    public:
     OnDestructionCallback() = default;
     OnDestructionCallback(OnDestructionCallback&&) = default;
@@ -232,18 +235,10 @@ class EventBase : public TimeoutManager,
     virtual void onEventBaseDestruction() noexcept = 0;
 
    private:
-    boost::intrusive::list_member_hook<
-        boost::intrusive::link_mode<boost::intrusive::normal_link>>
-        listHook_;
     Function<void(OnDestructionCallback&)> eraser_;
     Synchronized<bool> scheduled_{in_place, false};
 
-    using List = boost::intrusive::list<
-        OnDestructionCallback,
-        boost::intrusive::member_hook<
-            OnDestructionCallback,
-            decltype(listHook_),
-            &OnDestructionCallback::listHook_>>;
+    using List = boost::intrusive::list<OnDestructionCallback>;
 
     void schedule(
         FunctionRef<void(OnDestructionCallback&)> linker,
@@ -294,11 +289,11 @@ class EventBase : public TimeoutManager,
      * Factory function for creating the backend.
      */
     using BackendFactory =
-        folly::Function<std::unique_ptr<folly::EventBaseBackendBase>()>;
-    BackendFactory::SharedProxy backendFactory{nullptr};
+        std::function<std::unique_ptr<folly::EventBaseBackendBase>()>;
+    BackendFactory backendFactory{nullptr};
 
     Options& setBackendFactory(BackendFactory factoryFn) {
-      backendFactory = std::move(factoryFn).asSharedProxy();
+      backendFactory = std::move(factoryFn);
       return *this;
     }
 
@@ -1011,8 +1006,8 @@ class EventBase : public TimeoutManager,
   friend class detail::EventBaseLocalBase;
   template <typename T>
   friend class EventBaseLocal;
-  std::unordered_map<std::size_t, erased_unique_ptr> localStorage_;
-  folly::Synchronized<std::unordered_set<detail::EventBaseLocalBase*>>
+  folly::F14FastMap<std::size_t, erased_unique_ptr> localStorage_;
+  folly::Synchronized<folly::F14FastSet<detail::EventBaseLocalBase*>>
       localStorageToDtor_;
   bool tryDeregister(detail::EventBaseLocalBase&);
 
