@@ -97,7 +97,9 @@ class SSLContext {
     SSLv3,
     TLSv1, // support TLS 1.0+
     TLSv1_2, // support for only TLS 1.2+
+#if FOLLY_OPENSSL_HAS_TLS13
     TLSv1_3,
+#endif
   };
 
   /**
@@ -395,6 +397,17 @@ class SSLContext {
    * @param path Path to trusted certificate file
    */
   virtual void loadTrustedCertificates(const char* path);
+  /**
+   * Load trusted certificates from a vector of file paths
+   *
+   * @param paths container of file paths to trusted certificate files
+   */
+  template <typename StringList>
+  void loadTrustedCertificates(const StringList& paths) {
+    for (const auto& path : paths) {
+      loadTrustedCertificates(path.c_str());
+    }
+  }
   /**
    * Load trusted certificates from specified X509 certificate store.
    *
