@@ -118,7 +118,11 @@ struct Printer {
           }
         }
         toAppend(
-            v.asDouble(), &out_, opts_.double_mode, opts_.double_num_digits);
+            v.asDouble(),
+            &out_,
+            opts_.double_mode,
+            opts_.double_num_digits,
+            opts_.double_flags);
         break;
       case dynamic::INT64: {
         auto intval = v.asInt();
@@ -794,7 +798,7 @@ size_t firstEscapableInWord(T s, const serialization_opts& opts) {
       // Clear first 32 characters if this is the first index, since those are
       // always escaped.
       auto bitmap = opts.extra_ascii_to_escape_bitmap[i] &
-          (i == 0 ? uint64_t(-1) << 32 : ~0UL);
+          (i == 0 ? uint64_t(-1) << 32 : ~0ULL);
       while (bitmap) {
         auto bit = folly::findFirstSet(bitmap);
         needsEscape |= isChar(static_cast<uint8_t>(offset + bit - 1));
@@ -1002,14 +1006,18 @@ std::string stripComments(StringPiece jsonC) {
         result.push_back(s[0]);
         break;
       case State::InlineComment:
-        if (s.startsWith("*/")) {
+        if (s.startsWith('\n')) {
+          // preserve the line break to preserve the line count
+          result.push_back(s[0]);
+        } else if (s.startsWith("*/")) {
           state = State::None;
           ++i;
         }
         break;
       case State::LineComment:
         if (s[0] == '\n') {
-          // skip the line break. It doesn't matter.
+          // preserve the line break to preserve the line count
+          result.push_back(s[0]);
           state = State::None;
         }
         break;
