@@ -40,6 +40,19 @@ using namespace std::chrono_literals;
 XLOG_SET_CATEGORY_NAME("xlog_test.main_file")
 
 namespace {
+#if defined(__GNUC__) && !defined(__clang__) && __GNUC__ < 9
+// XLOG_SET_CATEGORY_NAME() does not work in gcc < 9.0
+// TODO: remove this when we drop support for gcc 8.x
+constexpr const char* current_xlog_category = "folly.logging.test.XlogTest.cpp";
+constexpr const char* current_xlog_category_original =
+    "folly/logging/test/XlogTest.cpp";
+constexpr const char* current_xlog_parent = "folly.logging.test.XlogTest";
+#else
+constexpr const char* current_xlog_category = "xlog_test.main_file";
+constexpr const char* current_xlog_category_original = "xlog_test.main_file";
+constexpr const char* current_xlog_parent = "xlog_test.main_file";
+#endif
+
 class XlogTest : public testing::Test {
  public:
   XlogTest() {
@@ -57,13 +70,13 @@ class XlogTest : public testing::Test {
 } // namespace
 
 TEST_F(XlogTest, xlogName) {
-  EXPECT_EQ("xlog_test.main_file", XLOG_GET_CATEGORY_NAME());
-  EXPECT_EQ("xlog_test.main_file", XLOG_GET_CATEGORY()->getName());
+  EXPECT_EQ(current_xlog_category_original, XLOG_GET_CATEGORY_NAME());
+  EXPECT_EQ(current_xlog_category, XLOG_GET_CATEGORY()->getName());
 }
 
 TEST_F(XlogTest, xlogIf) {
   auto handler = make_shared<TestLogHandler>();
-  LoggerDB::get().getCategory("xlog_test")->addHandler(handler);
+  LoggerDB::get().getCategory(current_xlog_parent)->addHandler(handler);
   auto& messages = handler->getMessages();
 
   // info messages are not enabled initially.
@@ -78,7 +91,7 @@ TEST_F(XlogTest, xlogIf) {
   messages.clear();
 
   // Increase the log level, then log a message.
-  LoggerDB::get().setLevel("xlog_test.main_file", LogLevel::DBG1);
+  LoggerDB::get().setLevel(current_xlog_category, LogLevel::DBG1);
   XLOG_IF(DBG1, false, "testing: ", 1, 2, 3);
   ASSERT_EQ(0, messages.size());
   messages.clear();
@@ -135,7 +148,7 @@ TEST_F(XlogTest, xlogIf) {
 
 TEST_F(XlogTest, xlog) {
   auto handler = make_shared<TestLogHandler>();
-  LoggerDB::get().getCategory("xlog_test")->addHandler(handler);
+  LoggerDB::get().getCategory(current_xlog_parent)->addHandler(handler);
   auto& messages = handler->getMessages();
 
   // info messages are not enabled initially.
@@ -146,7 +159,7 @@ TEST_F(XlogTest, xlog) {
   messages.clear();
 
   // Increase the log level, then log a message.
-  LoggerDB::get().setLevel("xlog_test.main_file", LogLevel::DBG1);
+  LoggerDB::get().setLevel(current_xlog_category, LogLevel::DBG1);
 
   XLOG(DBG1, "testing: ", 1, 2, 3);
   ASSERT_EQ(1, messages.size());
@@ -154,8 +167,8 @@ TEST_F(XlogTest, xlog) {
   EXPECT_TRUE(messages[0].first.getFileName().endsWith("XlogTest.cpp"))
       << "unexpected file name: " << messages[0].first.getFileName();
   EXPECT_EQ(LogLevel::DBG1, messages[0].first.getLevel());
-  EXPECT_EQ("xlog_test.main_file", messages[0].first.getCategory()->getName());
-  EXPECT_EQ("xlog_test", messages[0].second->getName());
+  EXPECT_EQ(current_xlog_category, messages[0].first.getCategory()->getName());
+  EXPECT_EQ(current_xlog_parent, messages[0].second->getName());
   messages.clear();
 
   XLOGF(WARN, "number: {:>3d}; string: {}", 12, "foo");
@@ -164,8 +177,8 @@ TEST_F(XlogTest, xlog) {
   EXPECT_TRUE(messages[0].first.getFileName().endsWith("XlogTest.cpp"))
       << "unexpected file name: " << messages[0].first.getFileName();
   EXPECT_EQ(LogLevel::WARN, messages[0].first.getLevel());
-  EXPECT_EQ("xlog_test.main_file", messages[0].first.getCategory()->getName());
-  EXPECT_EQ("xlog_test", messages[0].second->getName());
+  EXPECT_EQ(current_xlog_category, messages[0].first.getCategory()->getName());
+  EXPECT_EQ(current_xlog_parent, messages[0].second->getName());
   messages.clear();
 
   XLOG(DBG2, "this log check should not pass");
@@ -179,8 +192,8 @@ TEST_F(XlogTest, xlog) {
   EXPECT_TRUE(messages[0].first.getFileName().endsWith("XlogTest.cpp"))
       << "unexpected file name: " << messages[0].first.getFileName();
   EXPECT_EQ(LogLevel::INFO, messages[0].first.getLevel());
-  EXPECT_EQ("xlog_test.main_file", messages[0].first.getCategory()->getName());
-  EXPECT_EQ("xlog_test", messages[0].second->getName());
+  EXPECT_EQ(current_xlog_category, messages[0].first.getCategory()->getName());
+  EXPECT_EQ(current_xlog_parent, messages[0].second->getName());
   messages.clear();
 }
 
@@ -277,8 +290,8 @@ TEST_F(XlogTest, perFileCategoryHandling) {
 TEST_F(XlogTest, rateLimiting) {
   auto SEVEN = 7;
   auto handler = make_shared<TestLogHandler>();
-  LoggerDB::get().getCategory("xlog_test")->addHandler(handler);
-  LoggerDB::get().setLevel("xlog_test", LogLevel::DBG1);
+  LoggerDB::get().getCategory(current_xlog_parent)->addHandler(handler);
+  LoggerDB::get().setLevel(current_xlog_parent, LogLevel::DBG1);
 
   // Test XLOG_EVERY_N
   for (size_t n = 0; n < 50; ++n) {
@@ -451,8 +464,8 @@ TEST_F(XlogTest, rateLimiting) {
 
 TEST_F(XlogTest, rateLimitingEndOfThread) {
   auto handler = make_shared<TestLogHandler>();
-  LoggerDB::get().getCategory("xlog_test")->addHandler(handler);
-  LoggerDB::get().setLevel("xlog_test", LogLevel::DBG1);
+  LoggerDB::get().getCategory(current_xlog_parent)->addHandler(handler);
+  LoggerDB::get().setLevel(current_xlog_parent, LogLevel::DBG1);
 
   auto th = std::thread([&] {
     auto enqueue = [](int num) {
@@ -510,6 +523,7 @@ TEST_F(XlogTest, getXlogCategoryName) {
 
   // Buck's directory prefixes for generated source files
   // should be stripped out
+#ifdef FOLLY_XLOG_SUPPORT_BUCK1
   EXPECT_EQ(
       "myproject.generated_header.h",
       LogName::canonicalize(getXlogCategoryNameForFile(
@@ -518,6 +532,20 @@ TEST_F(XlogTest, getXlogCategoryName) {
       "foo.bar.test.h",
       LogName::canonicalize(getXlogCategoryNameForFile(
           "buck-out/gen/foo/bar#header-map,headers/foo/bar/test.h")));
+#endif // FOLLY_XLOG_SUPPORT_BUCK1
+
+#ifdef FOLLY_XLOG_SUPPORT_BUCK2
+  EXPECT_EQ(
+      "dirA.dirB.Foo.h",
+      LogName::canonicalize(getXlogCategoryNameForFile(
+          "buck-out/v2/gen/fbcode/bfcdfe5fb8e2f7e1/dirA/dirB/__some_lib_rule__/"
+          "buck-headers/dirA/dirB/Foo.h")));
+  EXPECT_EQ(
+      "abc.xyz.def.h",
+      LogName::canonicalize(getXlogCategoryNameForFile(
+          "buck-out/v2/gen/fbcode/bfcdfe5fb8e2f7e1/dirA/dirB/__some_lib_rule__/"
+          "buck-private-headers/abc/xyz/def.h")));
+#endif // FOLLY_XLOG_SUPPORT_BUCK2
 }
 
 TEST(Xlog, xlogStripFilename) {
