@@ -144,16 +144,16 @@ static_assert(
  *
  * Note that this is threadsafe.
  */
-#define XLOG_EVERY_MS_IF(level, cond, ms, ...)                             \
-  XLOG_IF(                                                                 \
-      level,                                                               \
-      (cond) &&                                                            \
-          [__folly_detail_xlog_ms = ms] {                                  \
-            static ::folly::logging::IntervalRateLimiter                   \
-                folly_detail_xlog_limiter(                                 \
-                    1, std::chrono::milliseconds(__folly_detail_xlog_ms)); \
-            return folly_detail_xlog_limiter.check();                      \
-          }(),                                                             \
+#define XLOG_EVERY_MS_IF(level, cond, ms, ...)                               \
+  XLOG_IF(                                                                   \
+      level,                                                                 \
+      (cond) &&                                                              \
+          [__folly_detail_xlog_ms = ms] {                                    \
+            static ::folly::logging::IntervalRateLimiter                     \
+                folly_detail_xlog_limiter(                                   \
+                    1, ::std::chrono::milliseconds(__folly_detail_xlog_ms)); \
+            return folly_detail_xlog_limiter.check();                        \
+          }(),                                                               \
       ##__VA_ARGS__)
 
 /**
@@ -162,17 +162,17 @@ static_assert(
  *
  * Note that this is threadsafe.
  */
-#define XLOGF_EVERY_MS(level, ms, fmt, arg1, ...)                      \
-  XLOGF_IF(                                                            \
-      level,                                                           \
-      [__folly_detail_xlog_ms = ms] {                                  \
-        static ::folly::logging::IntervalRateLimiter                   \
-            folly_detail_xlog_limiter(                                 \
-                1, std::chrono::milliseconds(__folly_detail_xlog_ms)); \
-        return folly_detail_xlog_limiter.check();                      \
-      }(),                                                             \
-      fmt,                                                             \
-      arg1,                                                            \
+#define XLOGF_EVERY_MS(level, ms, fmt, arg1, ...)                        \
+  XLOGF_IF(                                                              \
+      level,                                                             \
+      [__folly_detail_xlog_ms = ms] {                                    \
+        static ::folly::logging::IntervalRateLimiter                     \
+            folly_detail_xlog_limiter(                                   \
+                1, ::std::chrono::milliseconds(__folly_detail_xlog_ms)); \
+        return folly_detail_xlog_limiter.check();                        \
+      }(),                                                               \
+      fmt,                                                               \
+      arg1,                                                              \
       ##__VA_ARGS__)
 
 namespace folly {
@@ -307,14 +307,15 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogEveryNThreadImpl(size_t n) {
  *
  * The internal counters are process-global and threadsafe.
  */
-#define XLOG_N_PER_MS(level, count, ms, ...)                                   \
-  XLOG_IF(                                                                     \
-      level,                                                                   \
-      [] {                                                                     \
-        static ::folly::logging::IntervalRateLimiter                           \
-            folly_detail_xlog_limiter((count), std::chrono::milliseconds(ms)); \
-        return folly_detail_xlog_limiter.check();                              \
-      }(),                                                                     \
+#define XLOG_N_PER_MS(level, count, ms, ...)               \
+  XLOG_IF(                                                 \
+      level,                                               \
+      [] {                                                 \
+        static ::folly::logging::IntervalRateLimiter       \
+            folly_detail_xlog_limiter(                     \
+                (count), ::std::chrono::milliseconds(ms)); \
+        return folly_detail_xlog_limiter.check();          \
+      }(),                                                 \
       ##__VA_ARGS__)
 
 namespace folly {
@@ -357,10 +358,11 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  * though.)
  */
 #ifdef FOLLY_XLOG_STRIP_PREFIXES
-#define XLOG_FILENAME \
-  folly::xlogStripFilename(__FILE__, FOLLY_XLOG_STRIP_PREFIXES)
+#define XLOG_FILENAME        \
+  (static_cast<char const*>( \
+      ::folly::xlogStripFilename(__FILE__, FOLLY_XLOG_STRIP_PREFIXES)))
 #else
-#define XLOG_FILENAME __FILE__
+#define XLOG_FILENAME (static_cast<char const*>(__FILE__))
 #endif
 
 #define XLOG_IMPL(level, type, ...) \
@@ -413,25 +415,29 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  *   initialized.  On all subsequent calls, disabled log statements can be
  *   skipped with just a single check of the LogLevel.
  */
-#define XLOG_ACTUAL_IMPL(level, cond, always_fatal, type, ...)             \
-  (!XLOG_IS_ON_IMPL(level) || !(cond))                                     \
-      ? ::folly::logDisabledHelper(::folly::bool_constant<always_fatal>{}) \
-      : ::folly::LogStreamVoidify<::folly::isLogLevelFatal(level)>{} &     \
-          ::folly::LogStreamProcessor(                                     \
-              [] {                                                         \
-                static ::folly::XlogCategoryInfo<XLOG_IS_IN_HEADER_FILE>   \
-                    folly_detail_xlog_category;                            \
-                return folly_detail_xlog_category.getInfo(                 \
-                    &xlog_detail::xlogFileScopeInfo);                      \
-              }(),                                                         \
-              (level),                                                     \
-              xlog_detail::getXlogCategoryName(XLOG_FILENAME, 0),          \
-              xlog_detail::isXlogCategoryOverridden(0),                    \
-              XLOG_FILENAME,                                               \
-              __LINE__,                                                    \
-              __func__,                                                    \
-              (type),                                                      \
-              ##__VA_ARGS__)                                               \
+#define XLOG_ACTUAL_IMPL(level, cond, always_fatal, type, ...)              \
+  (!XLOG_IS_ON_IMPL(level) || !(cond))                                      \
+      ? ::folly::logDisabledHelper(::folly::bool_constant<always_fatal>{})  \
+      : ::folly::LogStreamVoidify<::folly::isLogLevelFatal(level)>{} &      \
+          ::folly::LogStreamProcessor(                                      \
+              [] {                                                          \
+                static ::folly::XlogCategoryInfo<XLOG_IS_IN_HEADER_FILE>    \
+                    folly_detail_xlog_category;                             \
+                return folly_detail_xlog_category.getInfo(                  \
+                    &::folly::detail::custom::xlogFileScopeInfo);           \
+              }(),                                                          \
+              (level),                                                      \
+              [] {                                                          \
+                constexpr auto* folly_detail_xlog_filename = XLOG_FILENAME; \
+                return ::folly::detail::custom::getXlogCategoryName(        \
+                    folly_detail_xlog_filename, 0);                         \
+              }(),                                                          \
+              ::folly::detail::custom::isXlogCategoryOverridden(0),         \
+              XLOG_FILENAME,                                                \
+              __LINE__,                                                     \
+              __func__,                                                     \
+              (type),                                                       \
+              ##__VA_ARGS__)                                                \
               .stream()
 
 /**
@@ -456,24 +462,28 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  *
  * See XlogLevelInfo for the implementation details.
  */
-#define XLOG_IS_ON_IMPL(level)                                \
-  ((level >= ::folly::LogLevel::FOLLY_XLOG_MIN_LEVEL) && [] { \
-    static ::folly::XlogLevelInfo<XLOG_IS_IN_HEADER_FILE>     \
-        folly_detail_xlog_level;                              \
-    return folly_detail_xlog_level.check(                     \
-        (level),                                              \
-        xlog_detail::getXlogCategoryName(XLOG_FILENAME, 0),   \
-        xlog_detail::isXlogCategoryOverridden(0),             \
-        &xlog_detail::xlogFileScopeInfo);                     \
+#define XLOG_IS_ON_IMPL(level)                                  \
+  ((level >= ::folly::LogLevel::FOLLY_XLOG_MIN_LEVEL) && [] {   \
+    static ::folly::XlogLevelInfo<XLOG_IS_IN_HEADER_FILE>       \
+        folly_detail_xlog_level;                                \
+    constexpr auto* folly_detail_xlog_filename = XLOG_FILENAME; \
+    constexpr folly::StringPiece folly_detail_xlog_category =   \
+        ::folly::detail::custom::getXlogCategoryName(           \
+            folly_detail_xlog_filename, 0);                     \
+    return folly_detail_xlog_level.check(                       \
+        (level),                                                \
+        folly_detail_xlog_category,                             \
+        ::folly::detail::custom::isXlogCategoryOverridden(0),   \
+        &::folly::detail::custom::xlogFileScopeInfo);           \
   }())
 
 /**
  * Get the name of the log category that will be used by XLOG() statements
  * in this file.
  */
-#define XLOG_GET_CATEGORY_NAME()                            \
-  (xlog_detail::isXlogCategoryOverridden(0)                 \
-       ? xlog_detail::getXlogCategoryName(XLOG_FILENAME, 0) \
+#define XLOG_GET_CATEGORY_NAME()                                        \
+  (::folly::detail::custom::isXlogCategoryOverridden(0)                 \
+       ? ::folly::detail::custom::getXlogCategoryName(XLOG_FILENAME, 0) \
        : ::folly::getXlogCategoryNameForFile(XLOG_FILENAME))
 
 /**
@@ -485,7 +495,7 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
  * expand to the correct filename based on where the macro is used.
  */
 #define XLOG_GET_CATEGORY() \
-  folly::LoggerDB::get().getCategory(XLOG_GET_CATEGORY_NAME())
+  (::folly::LoggerDB::get().getCategory(XLOG_GET_CATEGORY_NAME()))
 
 /**
  * XLOG_SET_CATEGORY_NAME() can be used to explicitly define the log category
@@ -508,17 +518,35 @@ FOLLY_EXPORT FOLLY_ALWAYS_INLINE bool xlogFirstNExactImpl(std::size_t n) {
 #define XLOG_SET_CATEGORY_CHECK
 #endif
 
-#define XLOG_SET_CATEGORY_NAME(category)                               \
-  namespace xlog_detail {                                              \
-  namespace {                                                          \
-  XLOG_SET_CATEGORY_CHECK                                              \
-  constexpr inline folly::StringPiece getXlogCategoryName(             \
-      folly::StringPiece, int) {                                       \
-    return category;                                                   \
-  }                                                                    \
-  constexpr inline bool isXlogCategoryOverridden(int) { return true; } \
-  }                                                                    \
+#if defined(__GNUC__) && !defined(__clang__) && __GNUC__ < 9
+// gcc 8.x crashes with an internal compiler error when trying to evaluate
+// getXlogCategoryName() in a constexpr expression.  Keeping category name
+// constexpr is important for performance, and XLOG_SET_CATEGORY_NAME() is
+// fairly rarely used, so simply let it be a no-op if compiling with older
+// versions of gcc.
+#define XLOG_SET_CATEGORY_NAME(category)
+#else
+#define XLOG_SET_CATEGORY_NAME(category)                                     \
+  namespace folly {                                                          \
+  namespace detail {                                                         \
+  namespace custom {                                                         \
+  namespace {                                                                \
+  struct xlog_correct_usage;                                                 \
+  static_assert(                                                             \
+      ::std::is_same<                                                        \
+          xlog_correct_usage,                                                \
+          ::folly::detail::custom::xlog_correct_usage>::value,               \
+      "XLOG_SET_CATEGORY_NAME() should not be used within namespace scope"); \
+  XLOG_SET_CATEGORY_CHECK                                                    \
+  FOLLY_CONSTEVAL inline StringPiece getXlogCategoryName(StringPiece, int) { \
+    return category;                                                         \
+  }                                                                          \
+  FOLLY_CONSTEVAL inline bool isXlogCategoryOverridden(int) { return true; } \
+  }                                                                          \
+  }                                                                          \
+  }                                                                          \
   }
+#endif
 
 /**
  * Assert that a condition is true.
@@ -636,8 +664,8 @@ namespace folly {
 
 class XlogFileScopeInfo {
  public:
-  std::atomic<::folly::LogLevel> level;
-  ::folly::LogCategory* category;
+  std::atomic<::folly::LogLevel> level{folly::LogLevel::UNINITIALIZED};
+  ::folly::LogCategory* category{nullptr};
 };
 
 /**
@@ -771,18 +799,18 @@ class XlogCategoryInfo<false> {
  */
 folly::StringPiece getXlogCategoryNameForFile(folly::StringPiece filename);
 
-constexpr bool xlogIsDirSeparator(char c) {
+FOLLY_CONSTEVAL bool xlogIsDirSeparator(char c) {
   return c == '/' || (kIsWindows && c == '\\');
 }
 
 namespace detail {
-constexpr const char* xlogStripFilenameRecursive(
+FOLLY_CONSTEVAL const char* xlogStripFilenameRecursive(
     const char* filename,
     const char* prefixes,
     size_t prefixIdx,
     size_t filenameIdx,
     bool match);
-constexpr const char* xlogStripFilenameMatchFound(
+FOLLY_CONSTEVAL const char* xlogStripFilenameMatchFound(
     const char* filename,
     const char* prefixes,
     size_t prefixIdx,
@@ -794,7 +822,7 @@ constexpr const char* xlogStripFilenameMatchFound(
                    filename, prefixes, prefixIdx, filenameIdx + 1)
              : (filename + filenameIdx));
 }
-constexpr const char* xlogStripFilenameRecursive(
+FOLLY_CONSTEVAL const char* xlogStripFilenameRecursive(
     const char* filename,
     const char* prefixes,
     size_t prefixIdx,
@@ -842,11 +870,12 @@ constexpr const char* xlogStripFilenameRecursive(
  * e.g., xlogStripFilename("/my/project/src/foo.cpp", "/tmp:/my/project")
  * would return "src/foo.cpp"
  */
-constexpr const char* xlogStripFilename(
+FOLLY_CONSTEVAL const char* xlogStripFilename(
     const char* filename, const char* prefixes) {
   return detail::xlogStripFilenameRecursive(filename, prefixes, 0, 0, true);
 }
-} // namespace folly
+
+namespace detail {
 
 /*
  * We intentionally use an unnamed namespace inside a header file here.
@@ -854,8 +883,10 @@ constexpr const char* xlogStripFilename(
  * We want each .cpp file that uses xlog.h to get its own separate
  * implementation of the following functions and variables.
  */
-namespace xlog_detail {
+namespace custom {
 namespace {
+struct xlog_correct_usage;
+
 /**
  * The default getXlogCategoryName() function.
  *
@@ -874,8 +905,8 @@ namespace {
  * over this one.
  */
 template <typename T>
-constexpr inline folly::StringPiece getXlogCategoryName(
-    folly::StringPiece filename, T) {
+FOLLY_CONSTEVAL inline StringPiece getXlogCategoryName(
+    StringPiece filename, T) {
   return filename;
 }
 
@@ -891,7 +922,7 @@ constexpr inline folly::StringPiece getXlogCategoryName(
  * over this one.
  */
 template <typename T>
-constexpr inline bool isXlogCategoryOverridden(T) {
+FOLLY_CONSTEVAL inline bool isXlogCategoryOverridden(T) {
   return false;
 }
 
@@ -903,6 +934,9 @@ constexpr inline bool isXlogCategoryOverridden(T) {
  * entire .cpp file, rather than needing a separate copy for each XLOG()
  * statement.
  */
-::folly::XlogFileScopeInfo xlogFileScopeInfo;
+FOLLY_CONSTINIT XlogFileScopeInfo xlogFileScopeInfo;
 } // namespace
-} // namespace xlog_detail
+} // namespace custom
+
+} // namespace detail
+} // namespace folly
