@@ -76,6 +76,7 @@
 #include <folly/ScopeGuard.h>
 #include <folly/Traits.h>
 #include <folly/Utility.h>
+#include <folly/lang/Access.h>
 #include <folly/lang/Exception.h>
 #include <folly/memory/MemoryResource.h>
 
@@ -375,6 +376,8 @@ class sorted_vector_set : detail::growth_policy_wrapper<GrowthPolicy> {
 
   Allocator get_allocator() const { return m_.cont_.get_allocator(); }
 
+  const Container& get_container() const noexcept { return m_.cont_; }
+
   sorted_vector_set& operator=(const sorted_vector_set& other) = default;
 
   sorted_vector_set& operator=(sorted_vector_set&& other) = default;
@@ -539,6 +542,54 @@ class sorted_vector_set : detail::growth_policy_wrapper<GrowthPolicy> {
     return find(key) == end() ? 0 : 1;
   }
 
+  std::pair<iterator, iterator> find(
+      const key_type& key1, const key_type& key2) {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  std::pair<const_iterator, const_iterator> find(
+      const key_type& key1, const key_type& key2) const {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  template <typename K>
+  std::pair<if_is_transparent<K, iterator>, if_is_transparent<K, iterator>>
+  find(const K& key1, const K& key2) {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  template <typename K>
+  std::pair<
+      if_is_transparent<K, const_iterator>,
+      if_is_transparent<K, const_iterator>>
+  find(const K& key1, const K& key2) const {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
   template <typename K>
   if_is_transparent<K, size_type> count(const K& key) const {
     return find(key) == end() ? 0 : 1;
@@ -687,6 +738,61 @@ class sorted_vector_set : detail::growth_policy_wrapper<GrowthPolicy> {
     }
     return end;
   }
+  template <typename Self, typename K>
+  static std::pair<self_iterator_t<Self>, self_iterator_t<Self>> lower_bound2_(
+      Self& self, K const& key1, K const& key2) {
+    auto len = self.size();
+    auto first = self.begin(), second = self.begin();
+    auto c = self.key_comp();
+    assert(!c(key2, key1));
+    while (true) {
+      if (len == 0) {
+        return std::make_pair(first, first);
+      }
+      auto half = len / 2;
+      auto middle = first + half;
+      if (c(*middle, key1)) {
+        first = middle + 1;
+        half = len - half - 1;
+      } else if (c(*middle, key2)) {
+        second = middle + (len & 1);
+        len = half;
+        break;
+      }
+      len = half;
+    }
+    while (len) {
+      auto half = len / 2;
+      auto middle1 = first + half;
+      auto middle2 = second + half;
+      if (c(*middle1, key1)) {
+        first = middle1 + (len & 1);
+      }
+      if (c(*middle2, key2)) {
+        second = middle2 + (len & 1);
+      }
+      len = half;
+    }
+    return std::make_pair(first, second);
+  }
+
+  template <typename Self, typename K>
+  static std::pair<self_iterator_t<Self>, self_iterator_t<Self>> find2_(
+      Self& self, K const& key1, K const& key2) {
+    auto end = self.end();
+    auto its = lower_bound2_(self, key1, key2);
+    if (its.second != end) {
+      if (self.key_comp()(key1, *its.first)) {
+        its.first = end;
+      }
+      if (self.key_comp()(key2, *its.second)) {
+        its.second = end;
+      }
+    } else if (its.first != end && self.key_comp()(key1, *its.first)) {
+      its.first = end;
+    }
+    return its;
+  }
 };
 
 // Swap function that can be found using ADL.
@@ -696,6 +802,15 @@ inline void swap(
   return a.swap(b);
 }
 
+template <typename T>
+struct is_sorted_vector_set : std::false_type {};
+
+template <typename... T>
+struct is_sorted_vector_set<folly::sorted_vector_set<T...>> : std::true_type {};
+
+template <typename T>
+constexpr bool is_sorted_vector_set_v = is_sorted_vector_set<T>::value;
+
 #if FOLLY_HAS_MEMORY_RESOURCE
 
 namespace pmr {
@@ -872,6 +987,8 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
 
   Allocator get_allocator() const { return m_.cont_.get_allocator(); }
 
+  const Container& get_container() const noexcept { return m_.cont_; }
+
   sorted_vector_map& operator=(const sorted_vector_map& other) = default;
 
   sorted_vector_map& operator=(sorted_vector_map&& other) = default;
@@ -997,6 +1114,16 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
     return insert(hint, std::move(value));
   }
 
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace(key_type&& k, Args&&... args) {
+    return try_emplace_impl(k, std::move(k), std::forward<Args>(args)...);
+  }
+
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace(const key_type& k, Args&&... args) {
+    return try_emplace_impl(k, k, std::forward<Args>(args)...);
+  }
+
   size_type erase(const key_type& key) {
     iterator it = find(key);
     if (it == end()) {
@@ -1034,6 +1161,54 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
     return find_(*this, key);
   }
 
+  std::pair<iterator, iterator> find(
+      const key_type& key1, const key_type& key2) {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  std::pair<const_iterator, const_iterator> find(
+      const key_type& key1, const key_type& key2) const {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  template <typename K>
+  std::pair<if_is_transparent<K, iterator>, if_is_transparent<K, iterator>>
+  find(const K& key1, const K& key2) {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
+  template <typename K>
+  std::pair<
+      if_is_transparent<K, const_iterator>,
+      if_is_transparent<K, const_iterator>>
+  find(const K& key1, const K& key2) const {
+    if (key_comp()(key2, key1)) {
+      auto iterators = find2_(*this, key2, key1);
+      access::swap(iterators.first, iterators.second);
+      return iterators;
+    } else {
+      return find2_(*this, key1, key2);
+    }
+  }
+
   mapped_type& at(const key_type& key) {
     iterator it = find(key);
     if (it != end()) {
@@ -1199,7 +1374,7 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
 
   template <typename Self, typename K>
   static self_iterator_t<Self> lower_bound(Self& self, K const& key) {
-    auto f = [c = self.key_comp()](value_type const& a, K const& b) {
+    auto f = [c = self.key_comp()](auto const& a, K const& b) {
       return c(a.first, b);
     };
     return std::lower_bound(self.begin(), self.end(), key, f);
@@ -1207,7 +1382,7 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
 
   template <typename Self, typename K>
   static self_iterator_t<Self> upper_bound(Self& self, K const& key) {
-    auto f = [c = self.key_comp()](K const& a, value_type const& b) {
+    auto f = [c = self.key_comp()](K const& a, auto const& b) {
       return c(a, b.first);
     };
     return std::upper_bound(self.begin(), self.end(), key, f);
@@ -1221,6 +1396,74 @@ class sorted_vector_map : detail::growth_policy_wrapper<GrowthPolicy> {
     // have to do this.
     return {lower_bound(self, key), upper_bound(self, key)};
   }
+
+  template <typename Self, typename K>
+  static std::pair<self_iterator_t<Self>, self_iterator_t<Self>> lower_bound2_(
+      Self& self, K const& key1, K const& key2) {
+    auto len = self.size();
+    auto first = self.begin(), second = self.begin();
+    auto c = self.key_comp();
+    assert(!c(key2, key1));
+    while (true) {
+      if (len == 0) {
+        return std::make_pair(first, first);
+      }
+      auto half = len / 2;
+      auto middle = first + half;
+      if (c(middle->first, key1)) {
+        first = middle + 1;
+        half = len - half - 1;
+      } else if (c(middle->first, key2)) {
+        second = middle + (len & 1);
+        len = half;
+        break;
+      }
+      len = half;
+    }
+    while (len) {
+      auto half = len / 2;
+      auto middle1 = first + half;
+      auto middle2 = second + half;
+      if (c(middle1->first, key1)) {
+        first = middle1 + (len & 1);
+      }
+      if (c(middle2->first, key2)) {
+        second = middle2 + (len & 1);
+      }
+      len = half;
+    }
+    return std::make_pair(first, second);
+  }
+
+  template <typename Self, typename K>
+  static std::pair<self_iterator_t<Self>, self_iterator_t<Self>> find2_(
+      Self& self, K const& key1, K const& key2) {
+    auto end = self.end();
+    auto its = lower_bound2_(self, key1, key2);
+    if (its.second != end) {
+      if (self.key_comp()(key1, its.first->first)) {
+        its.first = end;
+      }
+      if (self.key_comp()(key2, its.second->first)) {
+        its.second = end;
+      }
+    } else if (its.first != end && self.key_comp()(key1, its.first->first)) {
+      its.first = end;
+    }
+    return its;
+    ;
+  }
+
+  template <typename... Args>
+  std::pair<iterator, bool> try_emplace_impl(
+      const key_type& key, Args&&... args) {
+    iterator it = lower_bound(key);
+    if (it == end() || key_comp()(key, it->first)) {
+      return std::make_pair(
+          emplace_hint(it, std::forward<Args>(args)...), true);
+    }
+    return std::make_pair(it, false);
+  }
 };
 
 // Swap function that can be found using ADL.
@@ -1230,6 +1473,15 @@ inline void swap(
   return a.swap(b);
 }
 
+template <typename T>
+struct is_sorted_vector_map : std::false_type {};
+
+template <typename... T>
+struct is_sorted_vector_map<folly::sorted_vector_map<T...>> : std::true_type {};
+
+template <typename T>
+constexpr bool is_sorted_vector_map_v = is_sorted_vector_map<T>::value;
+
 #if FOLLY_HAS_MEMORY_RESOURCE
 
 namespace pmr {
