@@ -1004,6 +1004,25 @@ TEST(Singleton, LeakySingletonLSAN) {
   EXPECT_EQ(*ptr1, 1);
 }
 
+TEST(Singleton, LeakySingletonTSAN) {
+  struct PrivateTag {};
+  static folly::LeakySingleton<int, PrivateTag> gPtr;
+  auto* ptr0 = &gPtr.get();
+  EXPECT_EQ(*ptr0, 0);
+  auto func = [&]() {
+    auto val = gPtr.get();
+    EXPECT_TRUE(val == 0 || val == 1);
+  };
+  std::vector<std::thread> threads;
+  for (int i = 0; i < 100; i++) {
+    threads.emplace_back(func);
+  }
+  gPtr.make_mock([] { return new int(1); });
+  for (auto& t : threads) {
+    t.join();
+  }
+}
+
 TEST(Singleton, ShutdownTimer) {
   struct VaultTag {};
   struct PrivateTag {};
@@ -1023,16 +1042,12 @@ TEST(Singleton, ShutdownTimer) {
   vault.setShutdownTimeout(10ms);
   SingletonObject::try_get()->shutdownDuration = 10s;
   EXPECT_DEATH(
-      [&]() {
-        vault.startShutdownTimer();
-        vault.destroyInstances();
-      }(),
+      [&]() { vault.destroyInstancesFinal(); }(),
       "Failed to complete shutdown within 10ms.");
 
   vault.setShutdownTimeout(10s);
   SingletonObject::try_get()->shutdownDuration = 10ms;
-  vault.startShutdownTimer();
-  vault.destroyInstances();
+  vault.destroyInstancesFinal();
 }
 
 TEST(Singleton, ShutdownTimerDisable) {
@@ -1053,8 +1068,7 @@ TEST(Singleton, ShutdownTimerDisable) {
 
   vault.disableShutdownTimeout();
   SingletonObject::try_get()->shutdownDuration = 100ms;
-  vault.startShutdownTimer();
-  vault.destroyInstances();
+  vault.destroyInstancesFinal();
 }
 
 TEST(Singleton, ForkInChild) {
@@ -1155,3 +1169,21 @@ TEST(Singleton, EagerInitOnReenableSingletons) {
   EXPECT_EQ(4, counter1);
   EXPECT_EQ(2, counter2);
 }
+
+namespace {
+class CancelOnDestructionSingleton {
+ public:
+  ~CancelOnDestructionSingleton() {
+    CHECK(SingletonVault::singleton()
+              ->getDestructionCancellationToken()
+              .isCancellationRequested());
+  }
+};
+
+auto cancelOnDestructionSingleton =
+    folly::Singleton<CancelOnDestructionSingleton>{};
+} // namespace
+
+TEST(Singleton, CancelOnDestruction) {
+  cancelOnDestructionSingleton.try_get();
+}
