@@ -14,12 +14,18 @@
  * limitations under the License.
  */
 
+#include <functional>
 #include <iterator>
 #include <list>
+#include <map>
 #include <memory>
+#include <set>
 #include <string>
+#include <unordered_map>
+#include <unordered_set>
 #include <vector>
 
+#include <folly/Optional.h>
 #include <folly/Range.h>
 #include <folly/Utility.h>
 #include <folly/memory/Malloc.h>
@@ -33,6 +39,32 @@ using folly::sorted_vector_set;
 
 namespace {
 
+static_assert(
+    folly::is_sorted_vector_map_v<folly::sorted_vector_map<int, double>>);
+static_assert(
+    folly::is_sorted_vector_map_v<folly::sorted_vector_map<
+        int,
+        double,
+        /* Compare */ std::greater<int>,
+        /* Allocator */ std::allocator<std::pair<int, double>>,
+        /* GrowthPolicy */ void,
+        /* Container */ folly::small_vector<std::pair<int, double>, 10>>>);
+static_assert(!folly::is_sorted_vector_map_v<std::map<int, double>>);
+static_assert(!folly::is_sorted_vector_map_v<std::unordered_map<int, double>>);
+static_assert(
+    !folly::is_sorted_vector_map_v<std::vector<std::pair<int, double>>>);
+
+static_assert(folly::is_sorted_vector_set_v<folly::sorted_vector_set<int>>);
+static_assert(folly::is_sorted_vector_set_v<folly::sorted_vector_set<
+                  int,
+                  /* Compare */ std::greater<int>,
+                  /* Allocator */ std::allocator<int>,
+                  /* GrowthPolicy */ void,
+                  /* Container */ folly::small_vector<int, 20>>>);
+static_assert(!folly::is_sorted_vector_set_v<std::set<int>>);
+static_assert(!folly::is_sorted_vector_set_v<std::unordered_set<int>>);
+static_assert(!folly::is_sorted_vector_set_v<std::vector<int>>);
+
 template <class T>
 struct less_invert {
   bool operator()(const T& a, const T& b) const { return b < a; }
@@ -250,6 +282,16 @@ TEST(SortedVectorTypes, TransparentSetTest) {
   EXPECT_EQ(world, *s.find(world));
   EXPECT_TRUE(s.end() == s.find(zebra));
 
+  // find 2 keys
+  constexpr folly::StringPiece values[] = {buddy, hello, stake, world, zebra};
+  for (auto& v0 : values) {
+    for (auto& v1 : values) {
+      auto [it0, it1] = s.find(v0, v1);
+      EXPECT_TRUE(s.find(v0) == it0);
+      EXPECT_TRUE(s.find(v1) == it1);
+    }
+  }
+
   // count
   EXPECT_EQ(0, s.count(buddy));
   EXPECT_EQ(1, s.count(hello));
@@ -434,6 +476,43 @@ TEST(SortedVectorTypes, TransparentMapTest) {
   }
 }
 
+TEST(SortedVectorTypes, Find2) {
+  size_t sizes[] = {0, 1, 2, 31, 32, 33};
+  for (auto size : sizes) {
+    sorted_vector_set<int> s;
+    for (size_t i = 0; i < size; i++) {
+      s.insert(2 * i + 1);
+    }
+    // test find2 with every possible combination of pair of keys
+    for (size_t i = 0; i < 2 * size + 2; i++) {
+      for (size_t j = 0; j < 2 * size + 2; j++) {
+        auto expected0 = s.find(i);
+        auto expected1 = s.find(j);
+        auto [it0, it1] = s.find(i, j);
+        EXPECT_EQ(it0, expected0);
+        EXPECT_EQ(it1, expected1);
+      }
+    }
+  }
+
+  for (auto size : sizes) {
+    sorted_vector_map<int, int> m;
+    for (size_t i = 0; i < size; i++) {
+      m.emplace(2 * i + 1, i);
+    }
+    // test find2 with every possible combination of pair of keys
+    for (size_t i = 0; i < 2 * size + 2; i++) {
+      for (size_t j = 0; j < 2 * size + 2; j++) {
+        auto expected0 = m.find(i);
+        auto expected1 = m.find(j);
+        auto [it0, it1] = m.find(i, j);
+        EXPECT_EQ(it0, expected0);
+        EXPECT_EQ(it1, expected1);
+      }
+    }
+  }
+}
+
 TEST(SortedVectorTypes, Sizes) {
   EXPECT_EQ(sizeof(sorted_vector_set<int>), sizeof(std::vector<int>));
   EXPECT_EQ(
@@ -1088,6 +1167,7 @@ TEST(SortedVectorTypes, TestPmrMoveConstructSameAlloc) {
     auto d = s1.data();
 
     pmr::sorted_vector_set<int> s2(std::move(s1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(s1.get_allocator(), s2.get_allocator());
     EXPECT_EQ(s2.data(), d);
     EXPECT_EQ(s2.count(42), 1);
@@ -1099,6 +1179,7 @@ TEST(SortedVectorTypes, TestPmrMoveConstructSameAlloc) {
     auto d = m1.data();
 
     pmr::sorted_vector_map<int, int> m2(std::move(m1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_EQ(m1.get_allocator(), m2.get_allocator());
     EXPECT_EQ(m2.data(), d);
     EXPECT_EQ(m2.at(42), 42);
@@ -1120,6 +1201,7 @@ TEST(SortedVectorTypes, TestPmrMoveConstructDifferentAlloc) {
     auto d = s1.data();
 
     pmr::sorted_vector_set<int> s2(std::move(s1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_NE(s1.get_allocator(), s2.get_allocator());
     EXPECT_NE(s2.data(), d);
     EXPECT_EQ(s2.count(42), 1);
@@ -1131,6 +1213,7 @@ TEST(SortedVectorTypes, TestPmrMoveConstructDifferentAlloc) {
     auto d = m1.data();
 
     pmr::sorted_vector_map<int, int> m2(std::move(m1), a2);
+    // NOLINTNEXTLINE(bugprone-use-after-move)
     EXPECT_NE(m1.get_allocator(), m2.get_allocator());
     EXPECT_NE(m2.data(), d);
     EXPECT_EQ(m2.at(42), 42);
@@ -1291,3 +1374,48 @@ TEST(SortedVectorTypes, TestInsertHintCopy) {
   }
   EXPECT_EQ(CountCopyCtor::gCount_, 0);
 }
+
+TEST(SortedVectorTypes, TestTryEmplace) {
+  // folly::Optional becomes empty after move.
+  sorted_vector_map<folly::Optional<int>, folly::Optional<std::string>> map;
+  {
+    auto k = folly::make_optional<int>(1);
+    auto v = folly::make_optional<std::string>("1");
+    const auto& [it, inserted] = map.try_emplace(std::move(k), std::move(v));
+    EXPECT_TRUE(inserted);
+    EXPECT_EQ(it->first, 1);
+    EXPECT_EQ(it->second, "1");
+    EXPECT_FALSE(k);
+    EXPECT_FALSE(v);
+    EXPECT_EQ(map.size(), 1);
+  }
+  {
+    auto k = folly::make_optional<int>(1);
+    auto v = folly::make_optional<std::string>("another 1");
+    const auto& [it, inserted] = map.try_emplace(std::move(k), std::move(v));
+    EXPECT_FALSE(inserted);
+    EXPECT_EQ(it->first, 1);
+    EXPECT_EQ(it->second, "1");
+    EXPECT_EQ(k, 1);
+    EXPECT_EQ(v, "another 1");
+    EXPECT_EQ(map.size(), 1);
+  }
+  {
+    auto k = folly::make_optional<int>(2);
+    auto v = folly::make_optional<std::string>("2");
+    const auto& [it, inserted] = map.try_emplace(k, v);
+    EXPECT_TRUE(inserted);
+    EXPECT_EQ(it->first, 2);
+    EXPECT_EQ(it->second, "2");
+    EXPECT_EQ(k, 2);
+    EXPECT_EQ(v, "2");
+    EXPECT_EQ(map.size(), 2);
+  }
+}
+
+TEST(SortedVectorTypes, TestGetContainer) {
+  sorted_vector_set<int> set;
+  sorted_vector_map<int, int> map;
+  EXPECT_TRUE(set.get_container().empty());
+  EXPECT_TRUE(map.get_container().empty());
+}
